/*
**  S364.h - This header file contains common definitions
*/

/*
*******************************************************************************
**  Copyright 1998 Integrated Device Technology, Inc.
**  All Rights Reserved
*******************************************************************************
*/

#ifndef __S364134__
#define __S364134__
/******************************** D E F I N E S *******************************/
#ifndef GPIO_BASE
#ifdef S355
#define GPIO_BASE PHYS_TO_K1(0x18040000)
#else
#define GPIO_BASE PHYS_TO_K1(0x18000600)
#endif
#endif
/*
** following defines simple and uniform to save and restore context
** when enrtering and leaving as assemblu language program when memory
** and registers are both premiunm.
*/
#define SAVE_CNTXT    \
  subu  sp,64;    \
  sw  t0,60(sp);  \
  sw  t1,56(sp);  \
  sw  t2,52(sp);  \
  sw  t3,48(sp);  \
  sw  t4,44(sp);  \
  sw  t5,40(sp);  \
  sw  t6,36(sp);  \
  sw  t7,32(sp);  \
  sw  t8,28(sp);  \
  sw  t9,24(sp);  \
  sw  a0,20(sp);  \
  sw  a1,16(sp);  \
  sw  a2,12(sp);  \
  sw  a3,8(sp); \
  sw  ra,4(sp)

#define RSTR_CNTXT    \
  lw  t0,60(sp);  \
  lw  t1,56(sp);  \
  lw  t2,52(sp);  \
  lw  t3,48(sp);  \
  lw  t4,44(sp);  \
  lw  t5,40(sp);  \
  lw  t6,36(sp);  \
  lw  t7,32(sp);  \
  lw  t8,28(sp);  \
  lw  t9,24(sp);  \
  lw  a0,20(sp);  \
  lw  a1,16(sp);  \
  lw  a2,12(sp);  \
  lw  a3,8(sp); \
  lw  ra,4(sp); \
  add sp,64

/*
** Following define is to specify a maximum value for a software
** busy wait counter.
*/

#define LP_CNT_100NS  1000      /* set this based on processor speed */
#define LP_CNT_3S     1000000   /* set this based on processor speed */

/*
** Following are other common timer definitions.
*/
#ifdef S355
#define TIMER_BASE    PHYS_TO_K1(0x18028000)  
#define TIMEOUT_COUNT 0x00000FFF
#else
#define TIMER_BASE    PHYS_TO_K1(0x18000700)  
#endif
#define ENABLE_TIMER  0x1
#define DISABLE_TIMER 0x0
#define BIG_VALUE     0xFFFFFFFF

#ifdef S355
/* There is no DISPLAY on 355 boards*/
#elif defined(S334)
/*
** following few lines define a macro DISPLAY
** which is used to write a set of 4 characters
** onto the S334 LED.
*/

#if defined(EB332)
#define LED_BASE      PHYS_TO_K1(0x10000000)
#define LED_DIGIT0    0xc
#define LED_DIGIT1    0x8
#define LED_DIGIT2    0x4
#define LED_DIGIT3    0x0

#define LED_CLEAR     0x400

#else
#define LED_BASE      PHYS_TO_K1(0x14000000)
#define LED_DIGIT0    0xf
#define LED_DIGIT1    0xb
#define LED_DIGIT2    0x7
#define LED_DIGIT3    0x3

#define LED_CLEAR     0x400

#endif

#define DISPLAY(d0, d1, d2, d3)     \
        li    t6, LED_BASE                    ;\
        lb    t7, LED_CLEAR(t6)               ;\
              nop                             ;\
        li    t7, (d0) & 0xff                 ;\
        sb    t7, LED_DIGIT0(t6)              ;\
        li    t7, (d1) & 0xff                 ;\
        sb    t7, LED_DIGIT1(t6)              ;\
        li    t7, (d2) & 0xff                 ;\
        sb    t7, LED_DIGIT2(t6)              ;\
        li    t7, (d3) & 0xff                 ;\
        sb    t7, LED_DIGIT3(t6)

#define LEDCLEAR()              \
        li    t6, LED_BASE                    ;\
        lb    t7, LED_CLEAR(t6)               ;\
              nop

#endif

#define DESTRUCTIVE     1
#define NONDESTRUCTIVE  0

#endif
