/*
 * ocp.c
 *
 *	The is drived from pci.c
 *
 * 	Current Maintainer
 *      Armin Kuster akuster@pacbell.net
 *      Jan, 2002
 *
 *
 *
 * This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR   IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *	Version 1.0 (02/01/26) - A. Kuster
 *	Initial version	 -
 *
 *	Version 1.1 (04/19/02) - A. kuster
 *	added the ability to give # of emacs and return count
 *	added the addr & irq routines that were in *_enet.c
 *	improved ocp_register
 *	added ocp_alloc_dev
 *	created ocp_dev_count & get_ocp_dev
 *
 * 	Version 1.2 (4/22/02) - Armin
 * 	 added UART,I2C USB & IDE reg support
 * 	 vaddr is now ioremapped
 *
 * 	 Version 1.3 (04/25/02) - Armin/Todd
 * 	   cleanups
 * 	   bug fixes - ifndef's
 *	Vesrion 1.4 (05/03/02) - Armin
 *	    fixed ghost reg when ocp_reg fails , Idea from David Mueller
 *	    added ocp_get_max used by ocp_dev_count
 *	    ocp_reg  and get_ocp_dev call ocp_dev_count
 *	    removed need for ifndefs
 *	    Added use of core_ocp[]
 *	Version 1.5 - Armin
 *	added api descriptions
 *	name change to <class>_<action>_<obj>
 *	added ocp_type_info for name, short desciption and size
 *
 *	Version 1.5 - Armin/David Gibson
 *	added most of David ocp patch to cleanup some of the 
 *	APIs
 *	
 */

#include <linux/list.h>
#include <linux/module.h>
#include <linux/config.h>
#include <linux/miscdevice.h>
#include <linux/slab.h>
#include <linux/types.h>
#include <linux/init.h>
#include <linux/pm.h>
#include <linux/bootmem.h>
#include <asm/io.h>
#include <asm/ocp.h>
#include <asm/errno.h>
#ifdef CONFIG_85xx
#include <asm/rwsem.h>
#include <asm/semaphore.h>

extern int mem_init_done;
#endif /* CONFIG_85xx */

LIST_HEAD(ocp_list);
#ifdef CONFIG_85xx
LIST_HEAD(ocp_devices);                 /* List of all OCP devices */
LIST_HEAD(ocp_drivers);                 /* List of all OCP drivers */
DECLARE_RWSEM(ocp_devices_sem);         /* Global semaphores for those lists */
DECLARE_MUTEX(ocp_drivers_sem);         /* Global semaphores for those lists */

static int ocp_inited;
#endif /* CONFIG_85xx */

extern struct ocp_def core_ocp[];

struct type_info ocp_type_info[] = {
	{"pci", "PCI"},
	{"gpt", "General purpose timers"},
	{"uart", "NS16550 compatable serial device"},
	{"opb", "OBP"},
	{"I2C", "I2C bus"},
	{"gpio", "Gernal purpose I/O"},
	{"emac", "10/100 BaseT ethernet"},
	{"zmii", "ZMII bridge"},
	{"ide", "Independent Disk"},
	{"usb", "USB"},
	{"sci", "Smart card"},
	{"audio", "Sound interface"},
	{"ssp", "Sync serail port"},
	{"scp", "Serial Controller port"},
	{"scc", "Serial controller"},
	{"video", "General Video"},
	{"dma", "Dynamic memory allocation"},
	{"uic", "Universal Interrupt controller"},
	{"rtc", "Real Time Clock"},
	{"led", "Light Emitting diode"}

};

/**
 * ocp_get_max - get max for ocp type 
 * @type: OCP type such as PCI, GPT, UART, OPB, IIC, GPIO, EMAC, ZMII,
 *  
 * The routine returns max number of type that a given core supports
 */
int
ocp_get_max(int type)
{
	int max, i;
	for (i = 0, max = 0; core_ocp[i].type != OCP_NULL_TYPE; i++) {
		if (type == core_ocp[i].type)
			max++;
	}
	return (max);
}

/**
 * get_ocp_irq - get irq for ocp type and instance of it 
 * @type: OCP type such as PCI, GPT, UART, OPB, IIC, GPIO, EMAC, ZMII,
 * @dev_num: ocp device number whos paddr you want
 *  
 * The routine returns the valid irq, OCP_IRQ_NA or OCP_IRQ_MUL
 * of type and instance requested 
 */
int
ocp_get_irq(int type, int dev_num)
{
	int a, i;
	a = 0;

	for (i = 0; core_ocp[i].type != OCP_NULL_TYPE; i++) {
		if (type == core_ocp[i].type) {
			if (a == dev_num)
				return (core_ocp[i].irq);
			a++;

		}
	}

	return (-ENXIO);

}

/**
 * ocp_get_paddr - get phyical address for ocp type and instance of it 
 * @type: OCP type such as PCI, GPT, UART, OPB, IIC, GPIO, EMAC, ZMII
 * @dev_num: ocp device number whos paddr you want
 *  
 * The routine returns paddr of type requested and it's instance or
 * 0x0 on a failure
 */
unsigned long
ocp_get_paddr(int type, int dev_num)
{
	int a, i;
	a = 0;

	for (i = 0; core_ocp[i].type != OCP_NULL_TYPE; i++) {
		if (type == core_ocp[i].type) {
			if (a == dev_num)
				return ((unsigned long) core_ocp[i].paddr);
			a++;

		}
	}

	return 0;
}
/**
 * ocp_get_pm - get pm for ocp type and instance of it 
 * @type: OCP type such as PCI, GPT, UART, OPB, IIC, GPIO, EMAC, ZMII,
 * @dev_num: ocp device number whos power managment 
 * value you want
 *  
 * The routine returns the power management value or OCP_CPM_NA,
 * of type and instance requested 
 */
int
ocp_get_pm(int type, int dev_num)
{
	int a, i;
	a = 0;

	for (i = 0; core_ocp[i].type != OCP_NULL_TYPE; i++) {
		if (type == core_ocp[i].type) {
			if (a == dev_num)
				return (core_ocp[i].cpm);
			a++;

		}
	}

	return OCP_CPM_NA;
}

/* this sets the irq,paddr and anme to the device */
static int
ocp_set_dev(struct ocp_dev *ocp)
{
	int a, i;
	a = 0;

	strcpy(ocp->name, ocp_type_info[ocp->type].name);

	for (i = 0; core_ocp[i].type != OCP_NULL_TYPE; i++) {
		if (ocp->type == core_ocp[i].type) {
			if (a == ocp->num) {
				ocp->paddr = core_ocp[i].paddr;
				ocp->irq = core_ocp[i].irq;
				return 1;

			}
			a++;
		}
	}

	ocp->paddr = 0;
	ocp->irq = OCP_IRQ_NA;
	return 0;
}

/**
 * ocp_get_numtypes - This determines how many OCP devices of a givrn
 * type are registered
 * @type: OCP type such as PCI, GPT, UART, OPB, IIC, GPIO, EMAC, ZMII,
 *  
 * The routine returns number of types are registered
 */
static int
ocp_get_numtypes(int type)
{
	int count = 0;
	struct ocp_dev *ocp;
	struct list_head *ocp_l;

	list_for_each(ocp_l, &ocp_list) {
		ocp = list_entry(ocp_l, struct ocp_dev, ocp_list);
		if (type == ocp->type)
			count++;
	}

	return count;
}

/**
 * ocp_alloc_dev - alloc space for ocp driver
 * @sizeof_ocpdev: size of the device structure
 *  
 * The routine returns a valid 32 bit aligned
 * points the sizeof ocp_struct and the 
 * requested device structure
 */
struct ocp_dev *
ocp_alloc_dev(int sizeof_ocpdev)
{
	struct ocp_dev *drv;
	int alloc_size;

	/* ensure 32-byte alignment of the driver area */
	alloc_size = sizeof (*drv) + sizeof_ocpdev + 31;

	drv = (struct ocp_dev *) kmalloc(alloc_size, GFP_KERNEL);
	if (drv == NULL) {
		printk(KERN_ERR
		       "ocp_alloc_dev: Unable to allocate device memory.\n");
		return NULL;
	}

	memset(drv, 0, alloc_size);

	if (sizeof_ocpdev)
		drv->ocpdev = (void *) (((long) (drv + 1) + 31) & ~31);

	return (drv);
}

/**
 * ocp_free_dev - frees alloc space for ocp driver
 * @drv: the driver structure to register
 * 
 * Adds the driver structure to the list of registered drivers
 * Returns the number of ocp type devices which were claimed by the driver
 * during registration.  The driver remains registered even if the
 * return value is zero.
 */
/*
 * ocp_free_dev - frees alloc space for ocp driver
 * @dev: ocp device as allocated in ocp_alloc_dev
 */
void
ocp_free_dev(void *dev)
{
	kfree(dev);
}

/**
 * ocp_register - register a new ocp driver
 * @drv: the driver structure to register
 * 
 * Adds the driver structure to the list of registered drivers
 * Returns the number of ocp type devices which were claimed by the driver
 * during registration.  The driver remains registered even if the
 * return value is zero.
 */
int
ocp_register(struct ocp_dev *drv)
{
	int max;

	list_add(&drv->ocp_list, &ocp_list);
	max = ocp_get_max(drv->type);
	drv->num = ocp_get_numtypes(drv->type) - 1;

	if (drv->num >= max) {
		list_del(&drv->ocp_list);
		return -ENXIO;
	}

	if (!ocp_set_dev(drv)) {
		list_del(&drv->ocp_list);
		return -ENXIO;
	}
#ifdef CONFIG_OCP_PROC
	ocp_proc_attach_device(drv);
#endif
#ifdef OCP_DEBUG
	printk("Dev: %s Num:%d @ paddr:0x%x irq:%d\n", drv->name, drv->num,
	       drv->paddr, drv->irq);
#endif
	return (drv->num);
}

/**
 * ocp_unregister - unregister a ocp driver
 * @drv: the driver structure to unregister
 * 
 * Deletes the driver structure from the list of registered OCP drivers,
 * gives it a chance to clean up by calling its remove() function for
 * each device it was responsible for, and marks those devices as
 * driverless.
 */

void
ocp_unregister(struct ocp_dev *drv)
{
	list_del(&drv->ocp_list);
#ifdef CONFIG_OCP_PROC
	ocp_proc_detach_device(drv);
#endif
}

/**
 * ocp_get_dev - get ocp driver pointer for ocp type and instance of it 
 * @type: OCP type such as PCI, GPT, UART, OPB, IIC, GPIO, EMAC, ZMII
 * @dev_num: ocp device number whos paddr you want
 *  
 * The routine returns ocp device pointer 
 * in list based on type and instance of that device
 *
 */
struct ocp_dev *
ocp_get_dev(int type, int dev_num)
{
	struct ocp_dev *ocp;
	struct list_head *ocp_l;
	int count = 0;
	list_for_each(ocp_l, &ocp_list) {
		ocp = list_entry(ocp_l, struct ocp_dev, ocp_list);
		if (type == ocp->type) {
			if (dev_num == count)
				return ocp;
			count++;

		}
	}
	return NULL;
}

#ifdef CONFIG_85xx
/* Core of ocp_find_device(). Caller must hold ocp_devices_sem */
static struct ocp_device *
__ocp_find_device(unsigned int vendor, unsigned int function, int index)
{
        struct list_head        *entry;
        struct ocp_device       *dev, *found = NULL;

        list_for_each(entry, &ocp_devices) {
                dev = list_entry(entry, struct ocp_device, link);
                if (vendor != OCP_ANY_ID && vendor != dev->def->vendor)
                        continue;
                if (function != OCP_ANY_ID && function != dev->def->function)
                        continue;
                if (index != OCP_ANY_INDEX && index != dev->def->index)
                        continue;
                found = dev;
                break;
        }

        return found;
}

/**
 *      ocp_find_device -       Find a device by function & index
 *      @vendor: vendor ID of the device (or OCP_ANY_ID)
 *      @function: function code of the device (or OCP_ANY_ID)
 *      @idx: index of the device (or OCP_ANY_INDEX)
 *
 *      This function allows a lookup of a given function by it's
 *      index, it's typically used to find the MAL or ZMII associated
 *      with an EMAC or similar horrors.
 *      You can pass vendor, though you usually want OCP_ANY_ID there...
 */
struct ocp_device *
ocp_find_device(unsigned int vendor, unsigned int function, int index)
{
        struct ocp_device       *dev;

        down_read(&ocp_devices_sem);
        dev = __ocp_find_device(vendor, function, index);
        up_read(&ocp_devices_sem);

        return dev;
}

/**
 *      ocp_get_one_device -    Find a def by function & index
 *      @vendor: vendor ID of the device (or OCP_ANY_ID)
 *      @function: function code of the device (or OCP_ANY_ID)
 *      @idx: index of the device (or OCP_ANY_INDEX)
 *
 *      This function allows a lookup of a given ocp_def by it's
 *      vendor, function, and index.  The main purpose for is to
 *      allow modification of the def before binding to the driver
 */
struct ocp_def *
ocp_get_one_device(unsigned int vendor, unsigned int function, int index)
{
        struct ocp_device       *dev;
        struct ocp_def          *found = NULL;

        dev = ocp_find_device(vendor, function, index);

        if (dev)
                found = dev->def;

        return found;
}

/**
 *      ocp_add_one_device      -       Add a device
 *      @def: static device definition structure
 *
 *      This function adds a device definition to the
 *      device list. It may only be called before
 *      ocp_driver_init() and will return an error
 *      otherwise.
 */
int
ocp_add_one_device(struct ocp_def *def)
{
        struct  ocp_device      *dev;

        /* Can't be called after ocp driver init */
        if (ocp_inited)
                return 1;

        if (mem_init_done)
                dev = kmalloc(sizeof(*dev), GFP_KERNEL);
        else
                dev = alloc_bootmem(sizeof(*dev));

        if (dev == NULL)
                return 1;
        memset(dev, 0, sizeof(*dev));
        dev->def = def;
        dev->current_state = 4;
        sprintf(dev->name, "OCP device %04x:%04x:%04x",
                dev->def->vendor, dev->def->function, dev->def->index);
        down_write(&ocp_devices_sem);
        list_add_tail(&dev->link, &ocp_devices);
        up_write(&ocp_devices_sem);

        return 0;
}

/**
 *      ocp_remove_one_device - Remove a device by function & index
 *      @vendor: vendor ID of the device (or OCP_ANY_ID)
 *      @function: function code of the device (or OCP_ANY_ID)
 *      @idx: index of the device (or OCP_ANY_INDEX)
 *
 *      This function allows removal of a given function by its
 *      index. It may only be called before ocp_driver_init()
 *      and will return an error otherwise.
 */
int
ocp_remove_one_device(unsigned int vendor, unsigned int function, int index)
{
	struct ocp_device *dev;

        /* Can't be called after ocp driver init */
        if (ocp_inited)
                return 1;

        down_write(&ocp_devices_sem);
        dev = __ocp_find_device(vendor, function, index);
        list_del((struct list_head *)dev);
        up_write(&ocp_devices_sem);

        return 0;
}

/**
 *      ocp_early_init  -       Init OCP device management
 *
 *      This function builds the list of devices before setup_arch.
 *      This allows platform code to modify the device lists before
 *      they are bound to drivers (changes to paddr, removing devices
 *      etc)
 */
int __init
ocp_early_init(void)
{
        struct ocp_def  *def;

        /* Fill the devices list */
        for (def = core_ocp; def->vendor != OCP_VENDOR_INVALID; def++)
                ocp_add_one_device(def);

        return 0;
}

/**
 *      ocp_driver_init -       Init OCP device management
 *
 *      This function is meant to be called once, and only once to initialize
 *      the OCP device management. Note that it can actually be called at any
 *      time, it's perfectly legal to register drivers before
 *      ocp_driver_init() is called
 */
int
ocp_driver_init(void)
{
        /* ocp_driver_init is by default an initcall. If your arch requires
	 * this to be called earlier, then go on, ocp_driver_init is
	 * non-static for that purpose, and can safely be called twice
	 */
	if (ocp_inited)
                return 0;
        ocp_inited = 1;

        /* Call drivers probes */
        down(&ocp_drivers_sem);
        ocp_bind_drivers(NULL);
        up(&ocp_drivers_sem);

#ifdef CONFIG_PM
        pm_register(PM_SYS_DEV, 0, ocp_pm_callback);
#endif

        return 0;
}

/**
 *	ocp_driver_match	-	Match one driver to one device
 *	@drv: driver to match
 *	@dev: device to match
 *
 *	This function returns 0 if the driver and device don't match
 */
static int
ocp_driver_match(struct ocp_driver *drv, struct ocp_device *dev)
{
	const struct ocp_device_id *ids = drv->id_table;

	if (!ids)
		return 0;

	while (ids->vendor || ids->function) {
		if ((ids->vendor == OCP_ANY_ID
		     || ids->vendor == dev->def->vendor)
		    && (ids->function == OCP_ANY_ID
			|| ids->function == dev->def->function))
		        return 1;
		ids++;
	}
	return 0;
}


/**
 *	ocp_bind_drivers	-	Match all drivers with all devices
 *	@candidate: driver beeing registered
 *
 *	This function is called on driver registration and device discovery,
 *	it redo the matching of all "driverless" devices with all possible
 *	driver candidates.
 *	The driver beeing registered can be optionally passed in, in which
 *	case, the function will return -ENODEV is no match have been found
 *	or if all matches failed with a different code than -EAGAIN
 */
static int
ocp_bind_drivers(struct ocp_driver *candidate)
{
	struct list_head	*deventry, *drventry;
	struct ocp_device	*dev;
	struct ocp_driver	*drv;
	int			one_again, one_match;
	int			count = 0;

	do {
		/* We re-do the match loop if we had a sucess match and got one -EAGAIN
		 */
		one_match = one_again = 0;
		down_read(&ocp_devices_sem);
		list_for_each(deventry, &ocp_devices) {
			dev = list_entry(deventry, struct ocp_device, link);
			if (dev->driver != NULL)
				continue;
			list_for_each(drventry, &ocp_drivers) {
				drv = list_entry(drventry, struct ocp_driver, link);
				if (ocp_driver_match(drv, dev)) {
					int rc;

					/* Hrm... shall we set dev->driver after or before ? */
					rc = drv->probe(dev);
					if (rc == 0) {
						/* Driver matched, next device */
						dev->driver = drv;
						one_match = 1;
						if (drv == candidate)
							count++;
						break;
					} else if (rc == -EAGAIN) {
						/* Driver matched but asked for later call, next device */
						one_again = 1;
						if (drv == candidate)
							count++;
						break;
					}
				}
			}
		}
		up_read(&ocp_devices_sem);
	} while(one_match && one_again);

	return count;
}

/**
 *	ocp_register_driver	-	Register an OCP driver
 *	@drv: pointer to statically defined ocp_driver structure
 *
 *	The driver's probe() callback is called either recursively
 *	by this function or upon later call of ocp_driver_init
 *
 *      NOTE: Probe is called with ocp_drivers_sem held, it shouldn't
 *      call ocp_register/unregister_driver on his own code path.
 *      however, it _can_ call ocp_find_device().
 *
 *	NOTE2: Detection of devices is a 2 pass step on this implementation,
 *	hotswap isn't supported. First, all OCP devices are put in the device
 *	list, _then_ all drivers are probed on each match.
 *
 *      NOTE3: Drivers are allowed to return -EAGAIN from the probe() routine.
 *      this will cause them to be called again for this specific device as
 *	soon as another device have been probed or another driver registered.
 *	this, gives a simple way for a driver like EMAC to wait for another driver,
 *	like MAL to be up. There is potentially a small race if MAL happens to
 *	unregister, but this is hopefully never happening.
 *
 *	This function returns a count of how many devices actually matched
 *	(wether the probe routine returned 0 or -EAGAIN, a different error
 *	code isn't considered as a match).
 */

int
ocp_register_driver(struct ocp_driver *drv)
{
	int	rc = 0;

	/* Add to driver list */
	down(&ocp_drivers_sem);
	list_add_tail(&drv->link, &ocp_drivers);

	/* Check matching devices */
	rc = ocp_bind_drivers(drv);

	up(&ocp_drivers_sem);

	return rc;
}

/**
 *	ocp_unregister_driver	-	Unregister an OCP driver
 *	@drv: pointer to statically defined ocp_driver structure
 *
 *	The driver's remove() callback is called recursively
 *	by this function for any device already registered
 */

void
ocp_unregister_driver(struct ocp_driver *drv)
{
	struct ocp_device	*dev;
	struct list_head	*entry;

	/* Call remove() routine for all devices using it */
	down(&ocp_drivers_sem);
	down_read(&ocp_devices_sem);
	list_for_each(entry, &ocp_devices) {
		dev = list_entry(entry, struct ocp_device, link);
		if (dev->driver == drv) {
			drv->remove(dev);
			dev->driver = NULL;
			dev->drvdata = NULL;
		}
	}
	up_read(&ocp_devices_sem);

	/* Unlink driver structure */
	list_del_init(&drv->link);
	up(&ocp_drivers_sem);
}

__initcall(ocp_driver_init);
#endif /* CONFIG_85xx */

EXPORT_SYMBOL(ocp_get_irq);
EXPORT_SYMBOL(ocp_get_paddr);
EXPORT_SYMBOL(ocp_get_pm);
EXPORT_SYMBOL(ocp_get_max);
EXPORT_SYMBOL(ocp_get_dev);
EXPORT_SYMBOL(ocp_alloc_dev);
EXPORT_SYMBOL(ocp_free_dev);
EXPORT_SYMBOL(ocp_register);
EXPORT_SYMBOL(ocp_unregister);
#ifdef CONFIG_85xx
EXPORT_SYMBOL(ocp_find_device);
EXPORT_SYMBOL(ocp_register_driver);
EXPORT_SYMBOL(ocp_unregister_driver);
#endif /* CONFIG_85xx */
