/*
 * memec_2vpx.c
 *
 * Memec Virtex-II Pro 2VP7/2VP4 development board initialization
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * 2002-2004 (c) MontaVista, Software, Inc.  This file is licensed under the
 * terms of the GNU General Public License version 2.  This program is licensed
 * "as is" without any warranty of any kind, whether express or implied.
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/pagemap.h>
#include <linux/serial.h>
#include <linux/serialP.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/ocp.h>
#include <asm/keyboard.h>
#include <asm/pci-bridge.h>

#include "xilinx_ocp/xbasic_types.h"
extern u32 XWaitInAssert;
/*
 * The Xilinx drivers use something called XASSERT.  It should be
 * disabled in running systems to save time and space.  This is done by
 * defining NDEBUG when compiling anything that includes
 * xilinx_ocp/xbasic_types.h.  During development, it is handy to leave
 * NDEBUG off, but by default, the Xilinx code just tight loops.  In our
 * board_init function, we register a handler to be called that prints
 * out that we hit an XASSERT.  In addition, we tell it to not tight
 * loop by setting XWaitInAssert to zero.  This will probably lead to an
 * Oops and thus a backtrace.
 */

/* SAATODO: Make sure NDEBUG is on. */
void
reportXAssert(char *File, int Line)
{
	printk(KERN_CRIT "Xilinx OS Independent Code XAssert: %s:%d\n",
	       File, Line);
	printk(KERN_CRIT "Code may crash due to unhandled errors.\n");
}

/* Have OCP take care of the serial ports. */
struct ocp_def core_ocp[] = {
#ifdef XPAR_UARTNS550_0_BASEADDR
	{UART, XPAR_UARTNS550_0_BASEADDR, 31 - XPAR_INTC_0_UARTNS550_0_VEC_ID,
	 OCP_CPM_NA},
#ifdef XPAR_UARTNS550_1_BASEADDR
	{UART, XPAR_UARTNS550_1_BASEADDR, 31 - XPAR_INTC_0_UARTNS550_1_VEC_ID,
	 OCP_CPM_NA},
#ifdef XPAR_UARTNS550_2_BASEADDR
	{UART, XPAR_UARTNS550_2_BASEADDR, 31 - XPAR_INTC_0_UARTNS550_2_VEC_ID,
	 OCP_CPM_NA},
#ifdef XPAR_UARTNS550_3_BASEADDR
	{UART, XPAR_UARTNS550_3_BASEADDR, 31 - XPAR_INTC_0_UARTNS550_3_VEC_ID,
	 OCP_CPM_NA},
#ifdef XPAR_UARTNS550_4_BASEADDR
#error Edit this file to add more devices.
#endif				/* 4 */
#endif				/* 3 */
#endif				/* 2 */
#endif				/* 1 */
#endif				/* 0 */
	{OCP_NULL_TYPE, 0x0, OCP_IRQ_NA, OCP_CPM_NA}
};
#define NR_SER_PORTS (ARRAY_SIZE(core_ocp)-1)

/*
 * MTD flash stuff: two Toshiba TH50VSF2581 devices on P160 COMM module:
 * 2M*32 flash, and 256k*32 SRAM (total).
 */

#ifdef CONFIG_MTD

#define PHYS_PLASH_ADDR XPAR_FLASH_2MX32_MEM0_BASEADDR
#define PHYS_FLASH_SIZE (XPAR_FLASH_2MX32_MEM0_HIGHADDR - XPAR_FLASH_2MX32_MEM0_BASEADDR + 1)
#define FLASH_BUSWIDTH  4	/* 32 bits */

/* Reserve 512kB for bootinfo (e.g. MAC address) and/or bootloader */
#define FLASH_RESERVED_SIZE 0x80000

#ifdef CONFIG_MTD_PARTITIONS

#include <linux/mtd/physmap.h>

static struct mtd_partition memec2vpx_mtd_parts[] = {
	/* some space is reserved for bootinfo (e.g. MAC address) and/or bootloader */
	{
		.name =		"reserved",
		.size =		FLASH_RESERVED_SIZE,
		.offset =	0,
		.mask_flags =	MTD_WRITEABLE	/* force read-only */
	},
	/* the rest can be used for MTD */
	{
		.name =		"flashdisk",
		.size =		MTDPART_SIZ_FULL,
		.offset =	MTDPART_OFS_NXTBLK,
		.mask_flags =	0
	}
};

#define MTD_PARTITIONS_NUM (sizeof(memec2vpx_mtd_parts) / sizeof(struct mtd_partition))

#endif /* CONFIG_MTD_PARTITIONS */

#endif /* CONFIG_MTD */

/*
 * As an overview of how the following functions (board_init,
 * board_io_mapping and board_setup_arch) fit into the kernel startup
 * procedure, here's a call tree:
 *
 * start_here					  arch/ppc/kernel/head_4xx.S
 *   machine_init				  arch/ppc/kernel/setup.c
 *     platform_init				  arch/ppc/kernel/setup.c
 *   	 board_init				  this file
 *   MMU_init					  arch/ppc/mm/init.c
 *     *ppc_md.setup_io_mappings == m4xx_map_io	  arch/ppc/kernel/ppc4xx_setup.c
 *	 io_block_mapping of ONB and PCI
 *   	 board_io_mapping			  this file
 *   start_kernel				  init/main.c
 *     setup_arch				  arch/ppc/kernel/setup.c
 *   	 *ppc_md.setup_arch == ppc4xx_setup_arch  arch/ppc/kernel/ppc4xx_setup.c
 *   	   ppc4xx_find_bridges			  arch/ppc/kernel/ppc405_pci.c
 *   	   board_setup_arch			  this file
 */

void __init
board_init(void)
{
	XWaitInAssert = 0;
	XAssertSetCallback(reportXAssert);

#ifdef CONFIG_MTD
	/* we use generic physmap mapping driver and we use partitions */
	physmap_configure(PHYS_PLASH_ADDR, PHYS_FLASH_SIZE, FLASH_BUSWIDTH, NULL);
#ifdef CONFIG_MTD_PARTITIONS
	physmap_set_partitions(memec2vpx_mtd_parts, MTD_PARTITIONS_NUM);
#endif /* CONFIG_MTD_PARTITIONS */
#endif /* CONFIG_MTD */
}

void __init
board_io_mapping(void)
{
	extern struct serial_state rs_table[];
	int i;

	/* Remap all the serial ports */
	for (i = 0; i < NR_SER_PORTS; i++)
		rs_table[i].iomem_base =
		    ioremap((unsigned long) rs_table[i].iomem_base, 16);
}

void __init
board_setup_arch(void)
{
	/* Identify the system */
	printk(KERN_INFO "Memec Virtex-II Pro Development Board\n");
	printk(KERN_INFO "Port (C) 2002-2004 MontaVista Software, Inc. (source@mvista.com)\n");
}

/* Called after board_setup_irq from ppc4xx_init_IRQ(). */
void __init
board_setup_irq(void)
{
}
