
/*
 * $Id: jetway.c,v 1.1 2005/02/22 16:45:59 andyg Exp $
 *
 * Normal mappings of chips in physical memory
 *
 * Modified for Jetway partitions...
 */

/* partition support */
#define HAVE_PARTITIONS

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <asm/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>

#define BOOT_FLASH 		0xFFF80000
#define BOOT_FLASH_SIZE 	0x00080000
#define BOOT_BUSWIDTH 		1

#define APPL_FLASH		0x20000000
#define APPL_FLASH_SIZE		0x02000000
#define APPL_BUSWIDTH 		1

#define BOOT_OFFSET 0
#define BOOT_SIZE (512 * 1024)

#define USYSTEM_OFFSET 0
#define USYSTEM_SIZE (2 * 1024 * 1024)

#define FASTPATH_OFFSET (USYSTEM_OFFSET + USYSTEM_SIZE)
#define FASTPATH_SIZE (APPL_FLASH_SIZE/2 - FASTPATH_OFFSET)

#define EXTENDED_FLASH_OFFSET APPL_FLASH_SIZE/2
#define EXTENDED_FLASH_SIZE APPL_FLASH_SIZE/2

/* Helper structure */
typedef struct
{
  struct map_info *info_map;
  struct mtd_info **info_mtd;
  struct mtd_partition *info_part;
  long count_part;
  char *probe;
  long flash_start;
  long flash_size;
} myinfo_t;

struct map_info boot_map = {
name:"Jetway UBOOT",
size:BOOT_FLASH_SIZE,
buswidth:BOOT_BUSWIDTH,
};

struct map_info appl_map = {
name:"Jetway APPL",
size:APPL_FLASH_SIZE,
buswidth:APPL_BUSWIDTH,
};

static struct mtd_partition boot_partitions[] = {
{name: "Jetway UBOOT", size: BOOT_SIZE, offset:BOOT_OFFSET}
};

#define BOOT_PARTITIONS (sizeof(boot_partitions)/sizeof(boot_partitions[0]))

static struct mtd_partition appl_partitions[] = {
{name: "Jetway system (kernel + ramdisk)", size: USYSTEM_SIZE, offset:USYSTEM_OFFSET},
{name: "Jetway FastPath", size: FASTPATH_SIZE, offset:FASTPATH_OFFSET,},
{name: "Jetway extended flash disk", size: EXTENDED_FLASH_SIZE, offset:EXTENDED_FLASH_OFFSET},
};

#define APPL_PARTITIONS (sizeof(appl_partitions)/sizeof(appl_partitions[0]))

static struct mtd_info *boot_mtd = NULL;
static struct mtd_info *appl_mtd = NULL;

static myinfo_t myinfo[] = {
  {&boot_map, &boot_mtd, boot_partitions, BOOT_PARTITIONS, "jedec_probe", BOOT_FLASH,
   BOOT_FLASH_SIZE},
  {&appl_map, &appl_mtd, appl_partitions, APPL_PARTITIONS, "cfi_probe", APPL_FLASH,
   APPL_FLASH_SIZE},
};

int __init
init_jetway (void)
{
  int i, rc = 0;

  //printk ("init_jetway start\n");

  for (i = 0; i < sizeof (myinfo) / sizeof (myinfo[0]); i++)
  {
    myinfo_t *p = &myinfo[i];
    struct mtd_info *info_mtd;

    p->info_map->virt = (unsigned long) ioremap (p->flash_start, p->flash_size);
    if (!p->info_map->virt)
    {
      printk ("Failed to ioremap start = 0x%lx size = 0x%lx\n", p->flash_start, p->flash_size);
      rc = -EIO;
      continue;
    }

    //printk ("ioremap success start = 0x%lx size = 0x%lx\n", p->flash_start, p->flash_size);
    simple_map_init (p->info_map);

    info_mtd = do_map_probe (p->probe, p->info_map);
    if (!info_mtd)
    {
      printk ("Failed to probe map start = 0x%lx size = 0x%lx\n", p->flash_start, p->flash_size);
      iounmap ((void *) p->info_map->virt);
      rc = -ENXIO;
      continue;
    }

    //printk ("do_map_probe success start = 0x%lx size = 0x%lx\n", p->flash_start, p->flash_size);
    info_mtd->owner = THIS_MODULE;
#if defined(HAVE_PARTITIONS)
    add_mtd_partitions (info_mtd, p->info_part, p->count_part);
#else
    add_mtd_device (info_mtd);
#endif

    *p->info_mtd = info_mtd;
  }
  //printk ("init_jetway end\n");
  return rc;
}

static void __exit
cleanup_jetway (void)
{
  myinfo_t *p;

  //printk ("cleanup_jetway start\n");

  for (p = &myinfo[0]; p != NULL; p++)
  {
    struct mtd_info *info_mtd = *p->info_mtd;

    if (info_mtd)
    {
#if defined(HAVE_PARTITIONS)
      del_mtd_partitions (info_mtd);
#else
      del_mtd_device (info_mtd);
#endif
      map_destroy (info_mtd);
    }
    if (p->info_map->virt)
    {
      iounmap ((void *) p->info_map->virt);
      p->info_map->virt = 0;
    }
  }
  //printk ("cleanup_jetway end\n");
}

module_init (init_jetway);
module_exit (cleanup_jetway);

MODULE_LICENSE ("GPL");
MODULE_AUTHOR ("Rama Sasthri, Kristipati <rkristipati@lvl7.com>");
MODULE_DESCRIPTION ("MTD map driver for Vitesse Jetway Board");
