/*
 * (C) Copyright 2003
 * John W. Linville, LVL7 Systems, Inc., linville@lvl7.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <mpc824x.h>
#include <pci.h>
#include <devices.h>

#define SHIFT_AMOUNT 24
#define DEBUG_DRAM 0
#if DEBUG_DRAM
 #define DDRAM(fmt,args...) printf(fmt,##args)
#else 
 #define DDRAM(fmt,args...)
#endif 

int checkboard (void)
{

	printf("Board: A7301 IAM-B90-24G\n");

	return 0;
}

long int initdram (int board_type)
{

#if 1
       /* this code seems to successfully detect the difference between 
        * 128M and 256M or memory */

       int word, count, i;
       volatile ulong *addr;
       volatile ulong *base = CFG_SDRAM_BASE << SHIFT_AMOUNT;
       ulong save[1 << (32 - SHIFT_AMOUNT)];
       ulong shadow[1 << (32 - SHIFT_AMOUNT)]; 
       ulong ramsize = CFG_MAX_RAM_SIZE;
       register unsigned long val;

       /* lets start at 1M and run through all memory saving data */
       addr = base;
       count = 0;
       for (word = 0; (word << SHIFT_AMOUNT) < (CFG_MAX_RAM_SIZE); word++)
       {
         addr = (volatile ulong *)(word << SHIFT_AMOUNT);
         save[word] = *addr;
         shadow[word] = *addr;
         DDRAM("\nsetting save[%d] = shadow[%d] = 0x%lx at address 0x%x",word,word,*addr,addr);
         count++;
       }    

       for (i=count--; i < 1 << (32 - SHIFT_AMOUNT); i++)
         save[i] = shadow[i] = 0xffffffff;

       /* loop through every element, modify memory and shadow and
        * compare the 2, they should always be the same */
       for (; count >= 0; count--)
       {
         ulong tmp;
         /* invert the last address and the corresponding element in shadow */
         addr = (volatile long *)(count << SHIFT_AMOUNT);
         tmp = ~(*addr);
         *addr = tmp;
         shadow[count] = tmp;  

         DDRAM("\nsetting shadow[%d] = 0x%lx at 0x%lx",count,tmp,(ulong)addr); 

         /* repopulate save */
         for (word = 0; (word << SHIFT_AMOUNT) < (CFG_MAX_RAM_SIZE); word++)
         {
           addr = (volatile long *)(word << SHIFT_AMOUNT);
           save[word] = *addr;
           DDRAM("\nsetting save[%d] = 0x%lx",word,*addr); 
         }

         for (i=0;i< 1 << (32 - SHIFT_AMOUNT);i++)
         {
           if (shadow[i] != save[i])
           {
             DDRAM("\nshadow[%d] != save[%d] at 0x%x",i,i,i << SHIFT_AMOUNT);
             if ( ((i << SHIFT_AMOUNT) ^ (count << SHIFT_AMOUNT)) == 0x2000000)
             {
               ramsize = CFG_MAX_RAM_SIZE_128;
               break;
             }

             if ( ((i << SHIFT_AMOUNT) ^ (count << SHIFT_AMOUNT)) == 0x8000000 )  
             {
               ramsize = CFG_MAX_RAM_SIZE_256;
               break;
             }
            
           }
             
         }

       }

#else
       /* lets re-vrite these memory detection routines since I don't like them */

       int word, count, i;
       int j = 0;
       volatile ulong *addr;
       volatile ulong *base = CFG_SDRAM_BASE;
       ulong save[64];
       ulong shadow[64]; 
       ulong ramsize = 0;
       ulong tmpramsize = CFG_MAX_RAM_SIZE;
       register unsigned long val;

       /* lets fill save[] with values from 0 to 512M */ 
       for (i = 0; i < 64; i++)
       {
         addr = (volatile long *)(i << 23);
         save[i] = *addr;
         shadow[i] = *addr;
         DDRAM("\nsetting save[%d] = shadow[%d] = 0x%lx at 0x%lx",i,i,*addr,addr);
       }

       /* screw up the first entry */
       addr = (volatile long *)(j << 23);
       *addr = ~(save[j]);

       /* load save again */
       for (i = 0; i < 64; i++)
       {
         addr = (volatile long *)(i << 23);
         save[i] = *addr;
         DDRAM("\nsetting save[%d] = 0x%lx",i,*addr);
       }

       /* check to see that our set happened */
       if (shadow[j] == ~save[j]) {
         
         for (i = 0; i < 64; i++) {

           DDRAM("\n~save[%d] (0x%x) != shadow[%d] (0x%x)",j,~save[j],i,shadow[i]);

           /* if values match but they aren't the same address */
           if ((shadow[i] == ~save[j]) && (i != j)) {

             DDRAM("\n0x%x and 0x%x have the same value",j<<23,i<<23); 
          
             if (4 == i) {
               tmpramsize = CFG_MAX_RAM_SIZE_128;
               break;
             } else if (16 == i) { 
               tmpramsize = CFG_MAX_RAM_SIZE_256;
               break;
             }
           }
         }
       }
       else 
         printf("\n%s:%d something terrible has happened!",__FILE__,__LINE__);

       ramsize = tmpramsize;
#endif

       switch (ramsize)
       {
         case CFG_MAX_RAM_SIZE_128:
           /* reinitialize ppc8240 sdram controller */ 

           CONFIG_WRITE_WORD(MCCR1, (CFG_ROMNAL << MCCR1_ROMNAL_SHIFT) |
                                    (CFG_BANK0_ROW_128) |
                                    (CFG_BANK1_ROW_128 << MCCR1_BANK1ROW_SHIFT) |
                                    (CFG_BANK2_ROW << MCCR1_BANK2ROW_SHIFT) |
                                    (CFG_BANK3_ROW << MCCR1_BANK3ROW_SHIFT) |
                                    (CFG_BANK4_ROW << MCCR1_BANK4ROW_SHIFT) |
                                    (CFG_BANK5_ROW << MCCR1_BANK5ROW_SHIFT) |
                                    (CFG_BANK6_ROW << MCCR1_BANK6ROW_SHIFT) |
                                    (CFG_BANK7_ROW << MCCR1_BANK7ROW_SHIFT) |
                                    (CFG_ROMFAL << MCCR1_ROMFAL_SHIFT));
               
           CONFIG_WRITE_WORD(MSAR1,
               ( (CFG_BANK0_START_128 & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) |
               (((CFG_BANK1_START_128 & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 8) |
               (((CFG_BANK2_START & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 16) |
               (((CFG_BANK3_START & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(EMSAR1,
               ( (CFG_BANK0_START_128 & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) |
               (((CFG_BANK1_START_128 & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 8) |
               (((CFG_BANK2_START & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 16) |
               (((CFG_BANK3_START & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(MSAR2,
               ( (CFG_BANK4_START & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) |
               (((CFG_BANK5_START & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 8) |
               (((CFG_BANK6_START & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 16) |
               (((CFG_BANK7_START & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(EMSAR2,
               ( (CFG_BANK4_START & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) |
               (((CFG_BANK5_START & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 8) |
               (((CFG_BANK6_START & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 16) |
               (((CFG_BANK7_START & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(MEAR1,
               ( (CFG_BANK0_END_128 & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) |
               (((CFG_BANK1_END_128 & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 8) |
               (((CFG_BANK2_END & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 16) |
               (((CFG_BANK3_END & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(EMEAR1,
               ( (CFG_BANK0_END_128 & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) |
               (((CFG_BANK1_END_128 & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 8) |
               (((CFG_BANK2_END & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 16) |
               (((CFG_BANK3_END & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(MEAR2,
               ( (CFG_BANK4_END & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) |
               (((CFG_BANK5_END & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 8) |
               (((CFG_BANK6_END & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 16) |
               (((CFG_BANK7_END & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(EMEAR2,
               ( (CFG_BANK4_END & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) |
               (((CFG_BANK5_END & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 8) |
               (((CFG_BANK6_END & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 16) |
               (((CFG_BANK7_END & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 24));

           CONFIG_WRITE_BYTE(MBER,
                CFG_BANK0_ENABLE_128 |
               (CFG_BANK1_ENABLE_128 << 1) |
               (CFG_BANK2_ENABLE << 2) |
               (CFG_BANK3_ENABLE << 3) |
               (CFG_BANK4_ENABLE << 4) |
               (CFG_BANK5_ENABLE << 5) |
               (CFG_BANK6_ENABLE << 6) |
               (CFG_BANK7_ENABLE << 7));


           CONFIG_READ_WORD(MCCR1, val);
           CONFIG_WRITE_WORD(MCCR1, val | MCCR1_MEMGO); /* set memory access going */
           __asm__ __volatile__("eieio");

           return ramsize;
           break;
         case CFG_MAX_RAM_SIZE_256:
           /* reinitialize ppc8240 sdram controller */ 
           
           CONFIG_WRITE_WORD(MCCR1, (CFG_ROMNAL << MCCR1_ROMNAL_SHIFT) |
                                    (CFG_BANK0_ROW_256) |
                                    (CFG_BANK1_ROW_256 << MCCR1_BANK1ROW_SHIFT) |
                                    (CFG_BANK2_ROW << MCCR1_BANK2ROW_SHIFT) |
                                    (CFG_BANK3_ROW << MCCR1_BANK3ROW_SHIFT) |
                                    (CFG_BANK4_ROW << MCCR1_BANK4ROW_SHIFT) |
                                    (CFG_BANK5_ROW << MCCR1_BANK5ROW_SHIFT) |
                                    (CFG_BANK6_ROW << MCCR1_BANK6ROW_SHIFT) |
                                    (CFG_BANK7_ROW << MCCR1_BANK7ROW_SHIFT) |
                                    (CFG_ROMFAL << MCCR1_ROMFAL_SHIFT));
               
           CONFIG_WRITE_WORD(MSAR1,
               ( (CFG_BANK0_START_256 & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) |
               (((CFG_BANK1_START_256 & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 8) |
               (((CFG_BANK2_START & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 16) |
               (((CFG_BANK3_START & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(EMSAR1,
               ( (CFG_BANK0_START_256 & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) |
               (((CFG_BANK1_START_256 & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 8) |
               (((CFG_BANK2_START & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 16) |
               (((CFG_BANK3_START & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(MSAR2,
               ( (CFG_BANK4_START & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) |
               (((CFG_BANK5_START & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 8) |
               (((CFG_BANK6_START & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 16) |
               (((CFG_BANK7_START & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(EMSAR2,
               ( (CFG_BANK4_START & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) |
               (((CFG_BANK5_START & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 8) |
               (((CFG_BANK6_START & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 16) |
               (((CFG_BANK7_START & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(MEAR1,
               ( (CFG_BANK0_END_256 & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) |
               (((CFG_BANK1_END_256 & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 8) |
               (((CFG_BANK2_END & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 16) |
               (((CFG_BANK3_END & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(EMEAR1,
               ( (CFG_BANK0_END_256 & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) |
               (((CFG_BANK1_END_256 & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 8) |
               (((CFG_BANK2_END & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 16) |
               (((CFG_BANK3_END & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(MEAR2,
               ( (CFG_BANK4_END & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) |
               (((CFG_BANK5_END & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 8) |
               (((CFG_BANK6_END & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 16) |
               (((CFG_BANK7_END & MICR_ADDR_MASK) >> MICR_ADDR_SHIFT) << 24));
           CONFIG_WRITE_WORD(EMEAR2,
               ( (CFG_BANK4_END & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) |
               (((CFG_BANK5_END & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 8) |
               (((CFG_BANK6_END & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 16) |
               (((CFG_BANK7_END & MICR_EADDR_MASK) >> MICR_EADDR_SHIFT) << 24));

           CONFIG_WRITE_BYTE(MBER,
                CFG_BANK0_ENABLE_256 |
               (CFG_BANK1_ENABLE_256 << 1) |
               (CFG_BANK2_ENABLE << 2) |
               (CFG_BANK3_ENABLE << 3) |
               (CFG_BANK4_ENABLE << 4) |
               (CFG_BANK5_ENABLE << 5) |
               (CFG_BANK6_ENABLE << 6) |
               (CFG_BANK7_ENABLE << 7));

           CONFIG_READ_WORD(MCCR1, val);
           CONFIG_WRITE_WORD(MCCR1, val | MCCR1_MEMGO); /* set memory access going */
           __asm__ __volatile__("eieio");

           return ramsize;
           break;
         default:
           return CFG_MAX_RAM_SIZE;
       }
}

int board_pre_init(void)
{
#ifdef CONFIG_COM2_ENABLE
	uchar *val = (uchar *)(CFG_NS16550_COM1 + 0x11);

	/* Enable DUART mode... */
	*val = 1;
#endif

	/* Configurations for RCS2 and RCS3 */
	CONFIG_WRITE_WORD(ERCR1,0x94000000);
        /* changing this as recommended by delta 
	CONFIG_WRITE_WORD(ERCR2, 0x84000000); */

        CONFIG_WRITE_WORD(ERCR2,0x80af8000 | 
				0xa << 20 |  /* CFG_ROMFAL was 0x1f */
				CFG_ROMNAL << 15 | 
				CFG_ASRISE << 5  | 
				CFG_ASFALL << 10);
	CONFIG_WRITE_WORD(ERCR3,0x0400000e);
	CONFIG_WRITE_WORD(ERCR4,0x00000000);

        /* release switch chip from reset */
        *(uchar *)0x70000010 = 0;


        /* now we are going to enable REFINT errors in ERRENR1 */
        CONFIG_WRITE_BYTE(ERRENR1,0xff);
        CONFIG_WRITE_BYTE(ERRENR2,0xcf);
        
	return (0);
}

void null_putc(const char c)
{
	return;
}

void null_puts(const char *s)
{
	return;
}

int null_tstc(void)
{
	return (0);
}

int null_getc(void)
{
	return (0);
}

int misc_init_r(void)
{
	device_t nulldev;
	int rc;

	/* Create "null" device... */
	strcpy(nulldev.name, "null");
	nulldev.flags = DEV_FLAGS_INPUT | DEV_FLAGS_OUTPUT | DEV_FLAGS_SYSTEM;
	nulldev.putc = null_putc;
	nulldev.puts = null_puts;
	nulldev.getc = null_getc;
	nulldev.tstc = null_tstc;
	nulldev.start = NULL;

	rc = device_register(&nulldev);

	if (rc == 0) {

		char *s;

		s = getenv("stdin");
		if (s && strcmp(s, "null") == 0) {

			rc = console_assign(stdin, "null");
			if (rc != 0) return (rc);

		}

		s = getenv("stdout");
		if (s && strcmp(s, "null") == 0) {

			rc = console_assign(stdout, "null");
			if (rc != 0) return (rc);

		}

		s = getenv("stderr");
		if (s && strcmp(s, "null") == 0) {

			rc = console_assign(stderr, "null");
			if (rc != 0) return (rc);

		}

	}

	return (rc);
}

/*
 * Initialize PCI Devices, report devices found.
 */
#ifndef CONFIG_PCI_PNP
static struct pci_config_table pci_gsm74xx_config_table[] = {
	{ }
};
#endif

struct pci_controller hose = {
#ifndef CONFIG_PCI_PNP
	config_table: pci_gsm74xx_config_table,
#endif
};

void pci_init_board(void)
{
	pci_mpc824x_init(&hose);
}
