/*
 * (C) Copyright 2000
 * Rob Taylor, Flying Pig Systems. robt@flyingpig.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <mpc824x.h>
#include <pci.h>
#include "elmo_regs.h"

int checkboard (void)
{
	/*TODO: Check processor type */

	puts ("Board: Elmo 8241\n");
	return 0;
}

long int initdram (int board_type)
{
	int		 i;
	volatile uchar * addr;

    /* Do a memory detection test */
    for (i = 1; i <= CFG_MAX_RAM_SIZE; i<<=1) {
        addr = (uchar *)(i-1);
        *addr = 0xA5;
    }

    for (i = 1; i <= CFG_MAX_RAM_SIZE; i<<=1) {
        addr = (uchar *)(i-1);
        if (*addr != 0xA5) break;
    }
	
    return (i>>1);
}

int board_pre_init(void)
{
    uchar reg_value;
    uchar *pld_reg = NULL;

    uint eumb_value;
    uint *eumb_reg;
    
    /* Enable Inbound Translation Windows */
    (uchar *)eumb_reg = (uchar *)(CFG_EUMB_ADDR + 0x02310);
    *(uint *)eumb_reg = 0x1e000000;
    
    (uchar *)eumb_reg = (uchar *)(CFG_EUMB_ADDR + 0x02330);
    *(uint *)eumb_reg = 0x1e000000;

    /* Initialize ERCRs for Flash/PCI Access */
    CONFIG_WRITE_WORD( ERCR1, 0xb5050000);
    CONFIG_WRITE_WORD( ERCR2, 0x8dff8ce0);
    CONFIG_WRITE_WORD( ERCR3, 0x0000000d);
    CONFIG_WRITE_WORD( ERCR4, 0x08000004);

    /* With access to PLD Regs, we can do some other stuff */
    /* Disable watchdog timer */
    pld_reg = (uchar *)ELMO_REG_BASE + ENABLE_REG;
    reg_value = *pld_reg;
    
    reg_value &= ~ENABLE_REG_WD_EN;
    *pld_reg = reg_value;
    
    pld_reg = (uchar *)ELMO_REG_BASE + DEVICE_RSTS;
    reg_value = *pld_reg;
    
    reg_value &= ~DEVICE_RSTS_ALL;
    *pld_reg = reg_value;
    
    return 0;
}

/*
 * Initialize PCI Devices, report devices found.
 */
#ifndef CONFIG_PCI_PNP
static struct pci_config_table pci_elmo_config_table[] = {
	{ }
};
#endif

struct pci_controller hose = {
#ifndef CONFIG_PCI_PNP
	config_table: pci_elmo_config_table,
#endif
};

void pci_init_board(void)
{
    uchar reg_value;
    uchar *pld_reg = NULL;

    pld_reg = (uchar *)ELMO_REG_BASE + DEVICE_RSTS;
    reg_value = *pld_reg;
    
    reg_value &= ~DEVICE_RSTS_ALL;
    *pld_reg = reg_value;
    
	pci_mpc824x_init(&hose);
}
