/*
 * (C) Copyright 2003
 * John W. Linville, LVL7 Systems, Inc., linville@lvl7.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <mpc824x.h>
#include <pci.h>
#include <devices.h>

int checkreset(void)
{
	int rc;

	if (*((ulong *)CFG_LOAD_ADDR) != 0xdeadbeef) { /* check for prev boot */
		*((volatile ulong *)CFG_LOAD_ADDR) = 0xdeadbeef;
		flush_cache(0x00100000, CFG_CACHELINE_SIZE);
		*((volatile uchar *)0x70000020) = 0x83; /* magic reset */
	}

	if ((rc = display_options()) != 0) {
		return rc;
	}

	return checkcpu();
}

int checkboard(void)
{
	/* printf("Board: Netgear FSM73xx\n"); */
	/* printf("Board: Riverstone ES2010\n"); */

	return 0;
}

long int initdram(int board_type)
{
	int		 i, cnt;
	volatile uchar * base= CFG_SDRAM_BASE;
	volatile ulong * addr;
	ulong		 save[32];
	ulong		 val, ret  = 0;

	for (i=0, cnt=(CFG_MAX_RAM_SIZE / sizeof(long)) >> 1;
	     cnt > 0; cnt >>= 1) {
		addr = (volatile ulong *)base + cnt;
		save[i++] = *addr;
		*addr = ~cnt;
	}

	addr = (volatile ulong *)base;
	save[i] = *addr;
	*addr = 0;

	if (*addr != 0) {
		*addr = save[i];
		goto Done;
	}

	for (cnt = 1; cnt <= CFG_MAX_RAM_SIZE / sizeof(long); cnt <<= 1) {
		addr = (volatile ulong *)base + cnt;
		val = *addr;
		*addr = save[--i];
		if (val != ~cnt) {
			ret = cnt * sizeof(long);
			goto Done;
		}
	}

	ret = CFG_MAX_RAM_SIZE;
Done:
	return ret;
}

int board_pre_init(void)
{
#ifdef CONFIG_COM2_ENABLE
	uchar *val = (uchar *)(CFG_NS16550_COM1 + 0x11);

	/* Enable DUART mode... */
	*val = 1;
#endif

	/* Configurations for RCS2 and RCS3 */
	CONFIG_WRITE_WORD(0x800000d0, 0x84000000);
	CONFIG_WRITE_WORD(0x800000d4, 0x00000000);
	CONFIG_WRITE_WORD(0x800000d8, 0x00000000);

	return (0);
}

void null_putc(const char c)
{
	return;
}

void null_puts(const char *s)
{
	return;
}

int null_tstc(void)
{
	return (0);
}

int null_getc(void)
{
	return (0);
}

int misc_init_r(void)
{
	device_t nulldev;
	int rc;

	/* Create "null" device... */
	strcpy(nulldev.name, "null");
	nulldev.flags = DEV_FLAGS_INPUT | DEV_FLAGS_OUTPUT | DEV_FLAGS_SYSTEM;
	nulldev.putc = null_putc;
	nulldev.puts = null_puts;
	nulldev.getc = null_getc;
	nulldev.tstc = null_tstc;
	nulldev.start = NULL;

	rc = device_register(&nulldev);

	if (rc == 0) {

		char *s;

		s = getenv("stdin");
		if (s && strcmp(s, "null") == 0) {

			rc = console_assign(stdin, "null");
			if (rc != 0) return (rc);

		}

		s = getenv("stdout");
		if (s && strcmp(s, "null") == 0) {

			rc = console_assign(stdout, "null");
			if (rc != 0) return (rc);

		}

		s = getenv("stderr");
		if (s && strcmp(s, "null") == 0) {

			rc = console_assign(stderr, "null");
			if (rc != 0) return (rc);

		}

	}

	return (rc);
}

/*
 * Initialize PCI Devices, report devices found.
 */
#ifndef CONFIG_PCI_PNP
static struct pci_config_table pci_fsm74xx_config_table[] = {
	{ }
};
#endif

struct pci_controller hose = {
#ifndef CONFIG_PCI_PNP
	config_table: pci_fsm74xx_config_table,
#endif
};

void pci_init_board(void)
{
	pci_mpc824x_init(&hose);
}
