#include <common.h>
#include <malloc.h>
#include <net.h>
#include <asm/io.h>
#include <pci.h>

#if (CONFIG_COMMANDS & CFG_CMD_NET) && defined(CONFIG_NET_MULTI) && \
	defined(CONFIG_RTL8139) 


//#define PKTSIZE			1518 	: net.h
 
#define RTL_TIMEOUT 1
#define ETH_MAX_PACKET PKTSIZE
#define ETH_MIN_PACKET 64
#define ETHER_ADDR_SIZE 6

/* PCI Tuning Parameters
   Threshold is bytes transferred to chip before transmission starts. */
#define TX_FIFO_THRESH 256      /* In bytes, rounded down to 32 byte units. */
#define RX_FIFO_THRESH  4       /* Rx buffer level before first PCI xfer.  */
#define RX_DMA_BURST    4       /* Maximum PCI burst, '4' is 256 bytes */
#define TX_DMA_BURST    4       /* Calculate as 16<<val. */
#define NUM_TX_DESC     4       /* Number of Tx descriptor registers. */
#define TX_BUF_SIZE (ETH_MAX_PACKET-4)	/* FCS is added by the chip */
#define RX_BUF_LEN_IDX 0	/* 0, 1, 2 is allowed - 8,16,32K rx buffer */

#define RX_BUF_LEN (8192 << RX_BUF_LEN_IDX)

#if 0
#define DEBUG_TX
#define DEBUG_RX
#else
#undef DEBUG_TX
#undef DEBUG_RX
#endif


/* Symbolic offsets to registers. */
enum RTL8139_registers {
	MAC0=0,			/* Ethernet hardware address. */
	MAR0=8,			/* Multicast filter. */
	TxStatus0=0x10,		/* Transmit status (four 32bit registers). */
	TxAddr0=0x20,		/* Tx descriptors (also four 32bit). */
	RxBuf=0x30, RxEarlyCnt=0x34, RxEarlyStatus=0x36,
	ChipCmd=0x37, RxBufPtr=0x38, RxBufAddr=0x3A,
	IntrMask=0x3C, IntrStatus=0x3E,
	TxConfig=0x40, RxConfig=0x44,
	Timer=0x48,		/* general-purpose counter. */
	RxMissed=0x4C,		/* 24 bits valid, write clears. */
	Cfg9346=0x50, Config0=0x51, Config1=0x52,
	TimerIntrReg=0x54,	/* intr if gp counter reaches this value */
	MediaStatus=0x58,
	Config3=0x59,
	MultiIntr=0x5C,
	RevisionID=0x5E,	/* revision of the RTL8139 chip */
	TxSummary=0x60,
	MII_BMCR=0x62, MII_BMSR=0x64, NWayAdvert=0x66, NWayLPAR=0x68,
	NWayExpansion=0x6A,
	DisconnectCnt=0x6C, FalseCarrierCnt=0x6E,
	NWayTestReg=0x70,
	RxCnt=0x72,		/* packet received counter */
	CSCR=0x74,		/* chip status and configuration register */
	PhyParm1=0x78,TwisterParm=0x7c,PhyParm2=0x80,	/* undocumented */
	/* from 0x84 onwards are a number of power management/wakeup frame
	 * definitions we will probably never need to know about.  */
};

enum ChipCmdBits {
	CmdReset=0x10, CmdRxEnb=0x08, CmdTxEnb=0x04, RxBufEmpty=0x01, };

/* Interrupt register bits, using my own meaningful names. */
enum IntrStatusBits {
	PCIErr=0x8000, PCSTimeout=0x4000, CableLenChange= 0x2000,
	RxFIFOOver=0x40, RxUnderrun=0x20, RxOverflow=0x10,
	TxErr=0x08, TxOK=0x04, RxErr=0x02, RxOK=0x01,
};
enum TxStatusBits {
	TxHostOwns=0x2000, TxUnderrun=0x4000, TxStatOK=0x8000,
	TxOutOfWindow=0x20000000, TxAborted=0x40000000,
	TxCarrierLost=0x80000000,
};
enum RxStatusBits {
	RxMulticast=0x8000, RxPhysical=0x4000, RxBroadcast=0x2000,
	RxBadSymbol=0x0020, RxRunt=0x0010, RxTooLong=0x0008, RxCRCErr=0x0004,
	RxBadAlign=0x0002, RxStatusOK=0x0001,
};

enum MediaStatusBits {
	MSRTxFlowEnable=0x80, MSRRxFlowEnable=0x40, MSRSpeed10=0x08,
	MSRLinkFail=0x04, MSRRxPauseFlag=0x02, MSRTxPauseFlag=0x01,
};

enum MIIBMCRBits {
	BMCRReset=0x8000, BMCRSpeed100=0x2000, BMCRNWayEnable=0x1000,
	BMCRRestartNWay=0x0200, BMCRDuplex=0x0100,
};

enum CSCRBits {
	CSCR_LinkOKBit=0x0400, CSCR_LinkChangeBit=0x0800,
	CSCR_LinkStatusBits=0x0f000, CSCR_LinkDownOffCmd=0x003c0,
	CSCR_LinkDownCmd=0x0f3c0,
};

/* Bits in RxConfig. */
enum rx_mode_bits {
	RxCfgWrap=0x80,
	AcceptErr=0x20, 
	AcceptRunt=0x10, 
	AcceptBroadcast=0x08,
	AcceptMulticast=0x04, 
	AcceptMyPhys=0x02, 
	AcceptAllPhys=0x01,
};

enum Cfg9346Bits {
	Cfg9346_Lock = 0x00,
	Cfg9346_Unlock = 0xC0,
}; 

enum ClearBitMasks {
	MultiIntrClear = 0xF000,
	ChipCmdClear = 0xE2,
	Config1Clear = (1<<7)|(1<<6)|(1<<3)|(1<<2)|(1<<1),
};

static unsigned int cur_rx, cur_tx;

#define POINTER_TYPE

/* The RTL8139 can only transmit from a contiguous, aligned memory block.  */
#ifdef POINTER_TYPE
static unsigned char *tx_buffer;
static unsigned char *rx_ring;
#else
static unsigned char tx_butter[TX_BUF_SIZE] __attribute__((aligned(4)));
static unsigned char rx_ring[RX_BUF_LEN+16] __attribute__((aligned(4)));
#endif

static int rtl8139_probe (struct eth_device *dev, bd_t * bis);
static unsigned short read_eeprom(struct eth_device *dev, int location);
static void rtl_reset(struct eth_device *dev);
static int rtl_transmit (struct eth_device *dev, volatile void *packet, int length);
static int rtl_poll (struct eth_device *dev);
static void rtl_disable (struct eth_device *dev);

#define bus_to_phys(a)	pci_mem_to_phys((pci_dev_t)dev->priv, a)
#define phys_to_bus(a)	pci_phys_to_mem((pci_dev_t)dev->priv, a) 	

#define input_8(addr)			INB(addr)
#define input_le16(addr)		INW(addr)
#define input_le32(addr)		INL(addr)

#define output_8(addr, data)	OUTB(data, addr)
#define output_le16(addr, data)	OUTW(data, addr)
#define output_le32(addr, data)	OUTL(data, addr)

static u8 INB(u32 addr)
{
	return *(volatile u8 *)(addr);
}

static u16 INW(u32 addr)
{
	return le16_to_cpu(*(volatile u16 *)(addr));
}

static u32 INL(u32 addr)
{
	return le32_to_cpu(*(volatile u32 *)(addr));
}

static void OUTB(u8 command, u_long addr)
{
	*(volatile u8 *)(addr) = command;
}

static void OUTW(u16 command, u_long addr)
{
	*(volatile u16 *)(addr) = cpu_to_le16(command);
}

static void OUTL(u32 command, u_long addr)
{
	*(volatile u32 *)(addr) = cpu_to_le32(command);
}


static int rtl8139_probe (struct eth_device *dev, bd_t * bis)
{
	int i;
	int speed10, fullduplex;

	unsigned int rtladdr;
 
#ifdef POINTER_TYPE
	if (! tx_buffer)
	    tx_buffer = malloc(TX_BUF_SIZE + 16);
	if (! tx_buffer)
	    return 0;
	if (! rx_ring)
	    rx_ring = malloc(RX_BUF_LEN+32);
	if (! rx_ring)
	{
	    free(tx_buffer);
	    return 0;
	}

    /* Align */
	tx_buffer = (char *) ((16+(unsigned)tx_buffer) & ~15);
	rx_ring = (char *) ((16+(unsigned)rx_ring) & ~15);

#endif

	/* Mask the bit that says "this is an io addr" */
	rtladdr = dev->iobase;	

	for (i = 0; i < ETHER_ADDR_SIZE; i++) {		/*0-5*/
		printf("%02x", dev->enetaddr[i]);
		if (i < ETHER_ADDR_SIZE-1) printf(":");
	}

	speed10 = input_8(rtladdr + MediaStatus) & MSRSpeed10;

	fullduplex = input_le16(rtladdr + MII_BMCR) & BMCRDuplex;

	printf(" %sMbps %s-duplex\n", speed10 ? "10" : "100",
		fullduplex ? "full" : "half");

	rtl_reset(dev);

	if (input_8(rtladdr + MediaStatus) & MSRLinkFail) {
		printf("Cable not connected or other link failure\n");
		return(0);
	}

	return(1);
}

/* Serial EEPROM section. */

/*  EEPROM_Ctrl bits. */
#define EE_SHIFT_CLK    0x04    /* EEPROM shift clock. */
#define EE_CS           0x08    /* EEPROM chip select. */
#define EE_DATA_WRITE   0x02    /* EEPROM chip data in. */
#define EE_WRITE_0      0x00
#define EE_WRITE_1      0x02
#define EE_DATA_READ    0x01    /* EEPROM chip data out. */
#define EE_ENB          (0x80 | EE_CS)

/*
	Delay between EEPROM clock transitions.
	No extra delay is needed with 33Mhz PCI, but 66Mhz may change this.
*/

#define eeprom_delay()  input_le32(ee_addr); udelay(1)

/* The EEPROM commands include the alway-set leading bit. */
#define EE_WRITE_CMD    (5 << 6)	/* 101 000000 */
#define EE_READ_CMD     (6 << 6)	/* 110 000000 */
#define EE_ERASE_CMD    (7 << 6)	/* 111 000000 */

static unsigned short read_eeprom(struct eth_device *dev, int location)
{
	int i;
	unsigned int retval = 0;
	long ee_addr = dev->iobase + Cfg9346;
	int read_cmd = location | EE_READ_CMD;

	output_8(ee_addr, EE_ENB & ~EE_CS);		/* 0x80 */
	output_8(ee_addr, EE_ENB);				/* 0x80 | 0x08 */
	eeprom_delay();

	/* Shift the read command bits out. */
	for (i = 10; i >= 0; i--) {
		int dataval = (read_cmd & (1 << i)) ? EE_DATA_WRITE : 0;
		output_8(ee_addr, EE_ENB | dataval);
		eeprom_delay();
		output_8(ee_addr, EE_ENB | dataval | EE_SHIFT_CLK);
		eeprom_delay();
	}
	output_8(ee_addr, EE_ENB);
	eeprom_delay();

	for (i = 16; i > 0; i--) {
		output_8(ee_addr, EE_ENB | EE_SHIFT_CLK);
		eeprom_delay();
		retval = (retval << 1) | ((input_8(ee_addr) & EE_DATA_READ) ? 1 : 0);
		output_8(ee_addr, EE_ENB);
		eeprom_delay();
	}

	/* Terminate the EEPROM access. */
	output_8(ee_addr, ~EE_CS);		/* ~0x08 => 0xF7 */
	eeprom_delay();

	return retval;
}

static const unsigned int rtl8139_rx_config =
	 	(RX_FIFO_THRESH<<13) 	| 
		(RX_BUF_LEN_IDX<<11) 	| 
		(RX_DMA_BURST<<8);		

static void rtl_reset(struct eth_device *dev)
{
	int i;
	unsigned int rtladdr = dev->iobase;

	/* Soft reset the chip. */
	output_8(rtladdr+ChipCmd, (input_8(rtladdr+ChipCmd) & ChipCmdClear) | CmdReset  );

	cur_rx = 0;
	cur_tx = 0;

	/* Check that the chip has finished the reset. */
	for (i = 1000; i > 0; i--)
		if ((input_8(rtladdr + ChipCmd) & CmdReset) == 0)
			break;
		else 
			udelay(10);

	for (i = 0; i < 6; i++)
		output_8(rtladdr + MAC0 + i, dev->enetaddr[i]);

	/* Must enable Tx/Rx before setting transfer thresholds! */
	output_8(rtladdr + ChipCmd, CmdRxEnb | CmdTxEnb);
	output_le32(rtladdr + RxConfig,
		(RX_FIFO_THRESH<<13) | (RX_BUF_LEN_IDX<<11) | (RX_DMA_BURST<<8));		/* accept no frames yet!  */
	output_le32(rtladdr + TxConfig, (TX_DMA_BURST<<8)|0x03000000);

	/* The Linux driver changes Config1 here to use a different LED pattern
	 * for half duplex or full/autodetect duplex (for full/autodetect, the
	 * outputs are TX/RX, Link10/100, FULL, while for half duplex it uses
	 * TX/RX, Link100, Link10).  This is messy, because it doesn't match
	 * the inscription on the mounting bracket.  It should not be changed
	 * from the configuration EEPROM default, because the card manufacturer
	 * should have set that to match the card.  */
#if 0
#ifdef	DEBUG_RX
	printf("rx ring address is %p\n", rx_ring);
#endif
#endif
	output_le32(rtladdr + RxBuf, phys_to_bus((u32)rx_ring));

	/* If we add multicast support, the MAR0 register would have to be
	 * initialized to 0xffffffffffffffff (two 32 bit accesses).  Etherboot
	 * only needs broadcast (for ARP/RARP/BOOTP/DHCP) and unicast.  */
	output_8(rtladdr + ChipCmd, CmdRxEnb | CmdTxEnb);

	output_le32(rtladdr + RxConfig, rtl8139_rx_config);

	/* Start the chip's Tx and Rx process. */
	output_le32(rtladdr + RxMissed, 0);

	/* set_rx_mode */
	output_le32(rtladdr + RxConfig,	rtl8139_rx_config | AcceptBroadcast | AcceptMulticast | AcceptMyPhys);

	output_le32(rtladdr + MAR0 + 0, 0xffffffff);
	output_le32(rtladdr + MAR0 + 4, 0xffffffff);
	
	/* Disable all known interrupts by setting the interrupt mask. */
	output_le16(rtladdr + IntrMask, 0);

	/* Lock Config[01234] and BMCR register writes */
	output_8(rtladdr+Cfg9346, Cfg9346_Lock);
}


static int rtl_transmit (struct eth_device *dev, volatile void *packet, int length)
{
	int status = -1;

	unsigned int  to=0;
	unsigned int txstatus;
	unsigned int rtladdr = dev->iobase;
	unsigned int len = length;
#ifdef	DEBUG_TX
	int i;
#endif
	if (length <= 0) {
		printf ("%s: bad packet size: %d\n", dev->name, length);
		goto Done;
	}

	memcpy(tx_buffer, (unsigned char *)packet, len);

#ifdef	DEBUG_TX
	printf("\nsending %d bytes\n", len);
#endif 
          
	/* Note: RTL8139 doesn't auto-pad, send minimum payload (another 4
	 * bytes are sent automatically for the FCS, totalling to 64 bytes). */
	while (len < ETH_MIN_PACKET - 4) {	/* 64-4 = 60 */
		tx_buffer[len++] = '\0';
	}

	output_le32(rtladdr + TxAddr0 + cur_tx*4, phys_to_bus((u32)tx_buffer));
	output_le32(rtladdr + TxStatus0 + cur_tx*4,	((TX_FIFO_THRESH<<11) & 0x003f0000) | len);

	to = get_timer(0);
	do {
		status = input_le16(rtladdr + IntrStatus);	/* 3e*/
		/* Only acknlowledge interrupt sources we can properly handle
		 * here - the RxOverflow/RxFIFOOver MUST be handled in the
		 * rtl_poll() function.  */
		output_le16(rtladdr + IntrStatus, status & (TxOK | TxErr | PCIErr));
		if ((status & (TxOK | TxErr | PCIErr)) != 0) break;
	} while ( get_timer(to) <= 5000); /* */

	txstatus = input_le32(rtladdr+ TxStatus0 + cur_tx*4);

	if (status & TxOK) {
		cur_tx = ++cur_tx % NUM_TX_DESC;
#ifdef	DEBUG_TX
		printf("tx done (%d timeticks), status %x txstatus %X\n", to, status, txstatus);
#if 0
		for(i=0; i<len; i++) {
			printf("%02x ", tx_buffer[i]);
			if( (i!=0) && ((i%16) == 0) ) printf("\n");
		}
#endif
#endif
	} else {
#ifdef	DEBUG_TX
		printf("tx timeout/error (%d timeticks), status %x txstatus %X\n", to, status, txstatus);
#endif

		rtl_reset(dev);
	}

	status = len;

  Done:
	return status;

}

static unsigned char received_packet[2048] = {0,};

static int rtl_poll (struct eth_device *dev)
{
	unsigned int status;
	unsigned int ring_offs;
	unsigned int rx_size, rx_status;
	unsigned int len = 0, i;

	unsigned int rtladdr = dev->iobase;

	if (input_8(rtladdr + ChipCmd) & RxBufEmpty) {
		goto Done;
	}

	status = input_le16(rtladdr + IntrStatus);  /* 3E */
	/* See below for the rest of the interrupt acknowledges.  */
	output_le16(rtladdr + IntrStatus, status & ~(RxFIFOOver | RxOverflow | RxOK));

#ifdef	DEBUG_RX
	printf("\nrtl_poll: int %x ", status);
#endif

	ring_offs = cur_rx % RX_BUF_LEN;
	rx_status = le32_to_cpu(*(unsigned int*)(rx_ring + ring_offs));
	rx_size = rx_status >> 16;
	rx_status &= 0xffff;

	if ((rx_status & (RxBadSymbol|RxRunt|RxTooLong|RxCRCErr|RxBadAlign)) ||
	    (rx_size < ETH_MIN_PACKET) || (rx_size > ETH_MAX_PACKET)) {
		printf("rx error %x\n", rx_status);
		rtl_reset(dev);	/* this clears all interrupts still pending */
		goto Done;
	}

	/* Received a good packet */
	len = rx_size - 4;	/* no one cares about the FCS */
	if (ring_offs+4+rx_size-4 > RX_BUF_LEN) {
		int semi_count = RX_BUF_LEN - ring_offs - 4;

		memcpy(received_packet, rx_ring + ring_offs + 4, semi_count);
		memcpy(received_packet+semi_count, rx_ring, rx_size-4-semi_count);
#ifdef	DEBUG_RX
		printf("rx packet %d+%d bytes\n", semi_count,rx_size-4-semi_count);
#endif
	} else {
		memcpy(received_packet, rx_ring + ring_offs + 4, len);
#ifdef	DEBUG_RX
		printf("rx packet %d bytes\n", rx_size-4);
#if 0
		for(i=0; i<rx_size; i++) {
			printf("%02x ", received_packet[i]);
			if( (i!=0) && ((i%16) == 0) ) printf("\n");
		}
#endif
#endif
	}

	cur_rx = (cur_rx + rx_size + 4 + 3) & ~3;
	output_le16(rtladdr + RxBufPtr, cur_rx - 16);
	/* See RTL8139 Programming Guide V0.1 for the official handling of
	 * Rx overflow situations.  The document itself contains basically no
	 * usable information, except for a few exception handling rules.  */
	output_le16(rtladdr + IntrStatus, status & (RxFIFOOver | RxOverflow | RxOK));
	
	NetReceive(received_packet, len);
Done:
	return len;
}

static void rtl_disable (struct eth_device *dev)
{
	output_8(dev->iobase + ChipCmd, CmdReset);
}


/********************/
/* U-Boot interface */
/********************/
static struct pci_device_id supported[] = {
	{PCI_VENDOR_ID_REALTEK, PCI_DEVICE_ID_REALTEK_8139},
	{PCI_VENDOR_ID_REALTEK, PCI_DEVICE_ID_REALTEK_8129},
	{PCI_VENDOR_ID_REALTEK, PCI_DEVICE_ID_REALTEK_8029},
	{}
};

unsigned char MAC_ADDR[] = { 0x12, 0x34, 0x56, 0x78, 0x90, 0x00 };

static void read_hw_addr(struct eth_device *dev, bd_t *bis)
{
	unsigned short value;
	int i;

	/* read MAC address */
	value = read_eeprom(dev, 0);
	if (value != 0xffff && value != 0x0000 ) {
		for(i=0; i<3; i++) {
			value = read_eeprom(dev, i+7);
			dev->enetaddr[i*2]	=  value     & 0xFF;
			dev->enetaddr[i*2+1]= (value>>8) & 0xFF;
		}

	} else {
		for (i=0; i<6; i++) dev->enetaddr[i] = MAC_ADDR[i];
	}

	return;
}

int rtl8139_initialize (bd_t * bis)
{
	pci_dev_t devbusfn;

	int card_number = 0;
	struct eth_device *dev;
	u32 iobase, status;
	int idx = 0, i;
#if 0
	int cfrv;
#endif

	while (1) {
		/* Find PCI device */
		devbusfn =  pci_find_devices(supported, idx++);
		if (devbusfn == -1) {
			break;
		}
#if 0
		/* Get the chip configuration revision register. */
		pci_read_config_dword(devbusfn, PCI_REVISION_ID, &cfrv);
		cfrv &= 0xFF;
#endif
		pci_read_config_dword(devbusfn, PCI_COMMAND, &status);
		status |= (PCI_COMMAND_IO | PCI_COMMAND_MASTER);
		pci_write_config_dword(devbusfn, PCI_COMMAND, status);

		/* Check if I/O accesses and Bus Mastering are enabled.	 */
		pci_read_config_dword (devbusfn, PCI_COMMAND, &status);
		if (!(status & PCI_COMMAND_IO)) {
			printf ("Error: Can not enable IO access.\n");
			continue;
		}

		if (!(status & PCI_COMMAND_MASTER)) {
			printf ("Error: Can not enable Bus Mastering.\n");
			continue;
		}
 
		/* Set the latency timer for value. */
		pci_write_config_byte (devbusfn, PCI_LATENCY_TIMER, 0x20);

		/* read BAR for memory space access */
		pci_read_config_dword(devbusfn, PCI_BASE_ADDRESS_0, &iobase);
		iobase &= PCI_BASE_ADDRESS_IO_MASK;

		dev = (struct eth_device *) malloc (sizeof *dev);
		sprintf (dev->name, "rtl8139#%d ", card_number);

		dev->iobase = pci_io_to_phys(devbusfn, iobase); 
		dev->priv = (void *) devbusfn; /* this have to come before bus_to_phys() */
		dev->init = rtl8139_probe;
		dev->halt = rtl_disable;
		dev->send = rtl_transmit;
		dev->recv = rtl_poll;

		udelay (10 * 1000);

		/* Soft reset the chip. */
		output_8(dev->iobase+ChipCmd, (input_8(dev->iobase+ChipCmd) & ChipCmdClear) | CmdReset);

		/* Check that the chip has finished the reset. */
		for (i = 1000; i > 0; i--)
			if ((input_8(dev->iobase+ChipCmd) & CmdReset) == 0)
				break;
			else
				udelay (10);

		/* unlock Config[01234] and BMCR register writes */
		output_8(dev->iobase+Cfg9346, Cfg9346_Unlock);

		/* Bring the chip out of low-power mode. */
		output_8(dev->iobase + Config1, 0x00);

		read_hw_addr (dev, bis);

		eth_register(dev);

		card_number++;
	}

	return card_number;
}

#endif
