/*
 * (C) Copyright 2001, 2002
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2002
 * Frank Panno <fpanno@delphintech.com>, Delphin Technology AG
 *
 * Flash Routines for AMD device AM29DL323DB on the EP8260 board.
 *
 * This file is based on board/tqm8260/flash.c.
 *--------------------------------------------------------------------
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <mpc8xx.h>

#define V_ULONG(a)	(*(volatile unsigned long *)( a ))
#define V_BYTE(a)	(*(volatile unsigned char *)( a ))

/* Intel-compatible flash commands */
#define INTEL_PROGRAM         0x10   /* program */
#define INTEL_ERASE           0x20   /* erase */
#define INTEL_CLEAR           0x50   /* clear status register */
#define INTEL_WRITE_SETUP     0x40   /* write setup */
#define INTEL_LOCKBIT         0x60   /* lock bit */
#define INTEL_PROTECT         0x01   /* lock confirm */
#define INTEL_READID          0x90   /* read flash ID */
#define INTEL_CONFIRM         0xD0   /* confirm */
#define INTEL_ERASE_SUSPEND   0xB0   /* suspend erase */
#define INTEL_RESET           0xFF   /* reset to read mode */
#define INTEL_FINISHED        0x80   /* finished/OK */
#define INTEL_OK              0x80   /* finished/OK */

#ifndef CFG_FLASH_CFI_DRIVER

flash_info_t flash_info[CFG_MAX_FLASH_BANKS];


/*-----------------------------------------------------------------------
 */
void flash_reset(void)
{
  if( flash_info[0].flash_id != FLASH_UNKNOWN ) {
    V_ULONG( flash_info[0].start[0] ) = 0x00F000F0;
    V_ULONG( flash_info[0].start[0] + 4 ) = 0x00F000F0;
  }
}

/*-----------------------------------------------------------------------
 */
ulong flash_get_size( ulong baseaddr, flash_info_t *info )
{
#if 0
  short i;
  unsigned long flashtest_h, flashtest_l;
  /* Write auto select command sequence and test FLASH answer */
  V_BYTE(baseaddr + 0x55) = 0x98;

  flashtest_h = V_BYTE(baseaddr);		/* manufacturer ID	   */
  flashtest_l = V_BYTE(baseaddr + 1);

  printf("flashtest_h = 0x%x",flashtest_h);
  printf("flashtest_l = 0x%x",flashtest_l);

  if ((int)flashtest_h == INTEL_MANUFACT) {
	info->flash_id = FLASH_MAN_INTEL;
  } else {
	info->flash_id = FLASH_UNKNOWN;
	info->sector_count = 0;
	info->size = 0;
	return (0);  				/* no or unknown flash	   */
  }

  flashtest_h = V_ULONG(baseaddr + 8);	        /* device ID		   */
  flashtest_l = V_ULONG(baseaddr + 12);
  if (flashtest_h != flashtest_l) {
    info->flash_id = FLASH_UNKNOWN;
    return(0);
  }
  if (flashtest_h == AMD_ID_DL323B) {
    info->flash_id += FLASH_AMDL323B;
    info->sector_count = 71;
    info->size = 0x01000000;	                /* 4 * 4 MB = 16 MB	*/
  } else {
    info->flash_id = FLASH_UNKNOWN;
    return(0);  			 	/* no or unknown flash	   */
  }

  /* set up sector start adress table (bottom sector type) */
  for (i = 0; i < 8; i++) {
    info->start[i] = baseaddr + (i * 0x00008000);
  }
  for (i = 8; i < info->sector_count; i++) {
    info->start[i] = baseaddr + (i * 0x00040000) - 0x001C0000;
  }

  /* check for protected sectors */
  for (i = 0; i < info->sector_count; i++) {
    /* read sector protection at sector address, (A7 .. A0) = 0x02 */
    if ((V_ULONG( info->start[i] + 16 ) & 0x00010001) ||
	(V_ULONG( info->start[i] + 20 ) & 0x00010001)) {
	info->protect[i] = 1;		/* D0 = 1 if protected */
    } else {
	info->protect[i] = 0;
    }
  }

  flash_reset();
#else
  int i;

  info->flash_id = INTEL_ID_28F128J3;
  info->sector_count = CFG_MAX_FLASH_SECT;
  info->size = CFG_MAX_FLASH_SECT * CFG_FLASH_SECT_SIZE;

  for (i=0; i<CFG_MAX_FLASH_SECT; i++)
    info->start[i] = (CFG_FLASH_SECT_SIZE * i) + CFG_FLASH0_BASE;

#endif 

  return(info->size);
}

/*-----------------------------------------------------------------------
 */
unsigned long flash_init (void)
{
    unsigned long size_b0 = 0;
    int i;

    /* Init: no FLASHes known */
    for (i=0; i<CFG_MAX_FLASH_BANKS; ++i) {
	flash_info[i].flash_id = FLASH_UNKNOWN;
    }

    /* Static FLASH Bank configuration here (only one bank) */

    size_b0 = flash_get_size(CFG_FLASH0_BASE, &flash_info[0]);
    if (flash_info[0].flash_id == FLASH_UNKNOWN || size_b0 == 0) {
	printf ("## Unknown FLASH on Bank 0 - Size = 0x%08lx = %ld MB\n",
		size_b0, size_b0>>20);
    }

    /*
     * protect monitor and environment sectors
     */
#if 0
#if CFG_MONITOR_BASE >= CFG_FLASH0_BASE
    flash_protect(FLAG_PROTECT_SET,
		  CFG_MONITOR_BASE,
		  CFG_MONITOR_BASE + (2 * CFG_FLASH_SECT_SIZE) - 1,
		  &flash_info[0]);

printf("\nprotecting 0x%x for length 0x%x\n",CFG_MONITOR_BASE,(2 * CFG_FLASH_SECT_SIZE));

#endif

#if (CFG_ENV_IS_IN_FLASH == 1) && defined(CFG_ENV_ADDR)
# ifndef  CFG_ENV_SIZE
#  define CFG_ENV_SIZE	CFG_ENV_SECT_SIZE
# endif
    flash_protect(FLAG_PROTECT_SET,
		  CFG_ENV_ADDR,
		  CFG_ENV_ADDR + CFG_ENV_SIZE - 1,
		  &flash_info[0]);

printf("protecting 0x%x for length 0x%x\n",CFG_ENV_ADDR,CFG_ENV_SIZE);
#endif

    flash_protect(FLAG_PROTECT_SET,
		  flash_info[0].start[0],
		  flash_info[0].start[1] - 1,
		  &flash_info[0]);

printf("protecting 0x%x for length 0x%x\n",flash_info[0].start[0],flash_info[0].start[1]-flash_info[0].start[0] - 1);
#endif 
    return (size_b0);
}

/*-----------------------------------------------------------------------
 */
void flash_print_info  (flash_info_t *info)
{
    int i;

    if (info->flash_id == FLASH_UNKNOWN) {
	printf ("missing or unknown FLASH type\n");
	return;
    }

    switch ((info->flash_id >> 16) & 0xff) {
    case FLASH_MAN_AMD:	    printf ("AMD ");		    break;
    default:		    printf ("Unknown Vendor ");	    break;
    }

    switch (info->flash_id & FLASH_TYPEMASK) {
    case FLASH_AMDL323B:	printf ("29DL323B (32 M, bottom sector)\n");
				break;
    default:			printf ("Unknown Chip Type\n");
				break;
    }

    printf ("  Size: %ld MB in %d Sectors\n",
	    info->size >> 20, info->sector_count);

    printf ("  Sector Start Addresses:");
    for (i=0; i<info->sector_count; ++i) {
	if ((i % 5) == 0)
	  printf ("\n   ");
	printf (" %08lX%s",
		info->start[i],
		info->protect[i] ? " (RO)" : "     "
		);
    }
    printf ("\n");
    return;
}

/*-----------------------------------------------------------------------
 */
int flash_erase_intel (flash_info_t *info, int s_first, int s_last)
{
  int flag, sect;
  ulong start, now, last;
  int prot = 0;

  /* printf("Erasing 0x%x to 0x%x ",info->start[s_first],info->start[s_last+1]-1); */

  /* Validate arguments */
  if ((s_first < 0) || (s_first > s_last)) {
    if (info->flash_id == FLASH_UNKNOWN)
      printf ("- missing\n");
    else
      printf ("- no sectors to erase\n");
    return 1;

  }
  for (sect = s_first; sect <= s_last; sect++) 
  {
    vu_char *addr = (vu_char *)(info->start[sect]);     

    /* write 0x20, write 0xd0, wait for 0x80 and then write 0xff */ 
    *addr = INTEL_CLEAR;

    {int j; for(j=0;j<10000;) { j++; } } 

    *addr = INTEL_ERASE;

    {int j; for(j=0;j<10000;) { j++; } } 

    *addr = INTEL_CONFIRM;

    {int j; for(j=0;j<10000;) { j++; } } 
   
    while ((*addr & INTEL_FINISHED) != INTEL_FINISHED)
    {
      *addr = INTEL_CONFIRM;
      {int j; for(j=0;j<10000;) { j++; } } 
    }
    
    putc('.');

    *addr = INTEL_RESET;
  } 
  printf ("done!\n");
  return 0;
}

int flash_erase (flash_info_t *info, int s_first, int s_last)
{
  return flash_erase_intel(info,s_first,s_last);
}

static int write_byte_intel(flash_info_t *info, ulong dest, uchar data)
{
    volatile uchar *addr = (uchar *)dest, val;
    int rc = ERR_OK;
    int flag;
    int start;

    /* Check if Flash is (sufficiently) erased */
    if ((*addr & data) != data) {
        printf("Not erased: addr = %x *addr = %x, data = %x\n",
               addr, *addr, data);
        return (ERR_NOT_ERASED);
    }

    /* clear status register command */
    *addr = 0x50;

    {int j; for(j=0;j<10000;) { j++; } } 
    /* program set-up command */
    *addr = 0x40;

    {int j; for(j=0;j<10000;) { j++; } } 
    /* latch address/data */
    *addr = data;
    {int j; for(j=0;j<10000;) { j++; } } 

    while (((val = *addr) & 0x80) != 0x80) 
    {
      {int j; for(j=0;j<10000;) { j++; } } 
      
      /* printf("\n0x%x = 0x%x",addr,*addr); */ 
    }

    *addr = 0xFF; /* read array command */

#if 0 
    if(val & 0x1A) {    /* check for error */
        printf("\nFlash write error %02x at address %08lx\n",
               (int)val, (unsigned long)dest);
        if(val & (1<<3)) {
            printf("Voltage range error.\n");
            rc = ERR_PROG_ERROR;
        } else if(val & (1<<1)) {
            printf("Device protect error.\n");
            rc = ERR_PROTECTED;
        } else if(val & (1<<4)) {
            printf("Programming error.\n");
            rc = ERR_PROG_ERROR;
        }
        rc = ERR_PROG_ERROR;
    }

    *addr = 0xFF; /* read array command */
#endif 
    return rc;
}


/*-----------------------------------------------------------------------
 * Copy memory to flash, returns:
 * 0 - OK
 * 1 - write timeout
 * 2 - Flash not erased
 */

int write_buff (flash_info_t *info, uchar *src, ulong addr, ulong cnt)
{
#if 0
    ulong dp;
    static unsigned char bb[8];
    int i, l, rc, cc = cnt;

    dp = (addr & ~7);	/* get lower dword aligned address */

    /*
     * handle unaligned start bytes
     */
    if ((l = addr - dp) != 0) {
	for (i = 0; i < 8; i++)
	    bb[i] = (i < l || (i-l) >= cc) ? V_BYTE(dp+i) : *src++;
	if ((rc = write_dword(info, dp, bb)) != 0)
	{
	    return (rc);
	}
	dp += 8;
	cc -= 8 - l;
    }

    /*
     * handle word aligned part
     */
    while (cc >= 8) {
	if ((rc = write_dword(info, dp, src)) != 0) {
	    return (rc);
	}
	dp  += 8;
	src += 8;
	cc -= 8;
    }

    if (cc <= 0) {
	return (0);
    }

    /*
     * handle unaligned tail bytes
     */
    for (i = 0; i < 8; i++) {
	bb[i] = (i < cc) ? *src++ : V_BYTE(dp+i);
    }
    return (write_dword(info, dp, bb));
#else
    ulong start;
    int  rc;

    start = get_timer (0);

    /*
     * Write buffer to Flash
     */
    while (cnt > 0) {

        rc = write_byte_intel(info, addr, *src);

        if (rc != 0) {
            return (rc);
        }
        addr++;
        src++;
        cnt--;
        
        if (get_timer(start) > 1000) {   /* every second */
            putc ('.');
            start = get_timer(0);
        }
    }
  
    return (0);


#endif 
}

#endif
