#include <common.h>
#include <command.h>
#include <asm/iopin_8260.h>
#include <mii_phy.h>

#define MII_MDIO	0x01
#define MII_MDCK	0x02
#define MII_MDIR	0x04

# if defined(CONFIG_ETHER_PHY)

#if (CONFIG_COMMANDS & CFG_CMD_NET)

int do_phy_info (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
int do_phy_read (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
int do_phy_write (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
int do_phy_auto_neg (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);
int phy_auto_neg (unsigned short adr);

U_BOOT_CMD(
	pan,      2,     1,      do_phy_auto_neg,
	"pan      - Perform phy auto-negotiation\n",
	"[phy address]\n    - phy auto-negotiation\n"
);

U_BOOT_CMD(
//	phyinfo,      2,     1,      do_phy_info,
//	"phyinfo - ethernet phy information\n",
	pi,      2,     1,      do_phy_info,
	"pi      - ethernet phy information\n",
	"[phy address]\n    - phy information\n"
);

U_BOOT_CMD(
	pr,      3,     1,      do_phy_read,
	"pr      - read phy register\n",
	"[phy address] [phy register]\n    - phy read\n"
);

U_BOOT_CMD(
//	phywrite,      4,     1,      do_phy_write,
//	"phywrite - write phy register\n",
	pw,      4,     1,      do_phy_write,
	"pw      - write phy register\n",
	"[phy address] [phy register] [phy value]\n    - phy write\n"
);

int do_phy_info (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    unsigned short phy_addr;
    unsigned short phy_reg;
	unsigned short phy_index[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 16, 17, 18, 19, 24, 25, 26, 27, 28, 29, 30, 31, 0xFFFF};
	unsigned short i;

	if(argc == 1)
		phy_addr = 3;
	else
		phy_addr = simple_strtoul(argv[1], NULL, 16);
	printf("-----[Phy Register Dump]-----\n");
	printf("Phy Address %d    Phy Address %d\n", phy_addr, phy_addr+1);
	for(i=0; phy_index[i] != 0xFFFF; i++) {
		phy_reg = mii_phy_read(phy_addr, phy_index[i]);
		printf(" %2d - 0x%04x      ", phy_index[i], phy_reg);
		phy_reg = mii_phy_read(phy_addr+1, phy_index[i]);
		printf("%2d - 0x%04x\n", phy_index[i], phy_reg);
	}
	return 0;
}

int do_phy_read (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    unsigned short phy_addr;
    unsigned short phy_index;
    unsigned short phy_data;

	if (argc == 3) {
		phy_addr = simple_strtoul(argv[1], NULL, 16);
		phy_index = simple_strtoul(argv[2], NULL, 10);
		phy_data = mii_phy_read(phy_addr, phy_index);
		printf("%2d - 0x%04x\n", phy_index, phy_data);
	}
	return 0;
}

int do_phy_write (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    unsigned short phy_addr;
    unsigned short phy_index;
    unsigned short phy_data;

	if (argc == 4) {
		phy_addr = simple_strtoul(argv[1], NULL, 16);
		phy_index = simple_strtoul(argv[2], NULL, 10);
		phy_data = simple_strtoul(argv[3], NULL, 16);
		mii_phy_write(phy_addr, phy_index, phy_data);
	}
	return 0;
}

int do_phy_auto_neg (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
    unsigned short phy_addr;

	if(argc == 1)
		phy_addr = 0;
	else
		phy_addr = simple_strtoul(argv[1], NULL, 16);

	phy_auto_neg(phy_addr);
	return 0;
}

int phy_auto_neg (unsigned short phy_adr)
{
	if(mii_phy_read(phy_adr, 0) == 0xFFFF) {
		printf("ERROR: No Phy present at address %d.\n");
	}
	else {
		mii_phy_write(phy_adr, 4, 0x5E1);
		mii_phy_write(phy_adr, 0, 0x1200);
		while((mii_phy_read(phy_adr, 0) & 0x0200) != 0);
		mii_phy_write(phy_adr, 20, 0x0D52);
	}
	return 0;
}

void reset_phy (void)
{
//	phy_auto_neg (0);
//	phy_auto_neg (1);
}

void
mii_discover_phy(int phy_addr)
{
    int known;
    unsigned short phy_reg;
    unsigned long phy_id;

    phy_addr += 3;

    known = 0;
    printf("Discovering phy @ %d: ", phy_addr);
    phy_id = mii_phy_read(phy_addr, 2) << 16;
    phy_id |= mii_phy_read(phy_addr, 3);
    if ((phy_id & 0xFFFFFC00) == 0x00406000) {
	printf("Level One ");
	if ((phy_id & 0x000003F0) == 0x320) {
	    printf("BCM5222 Revision %d\n", (int)(phy_id & 0xF));
	    known = 1;
	}
	else printf("unknown type\n");
    }
    else printf("unknown OUI = 0x%08lX\n", phy_id);

    phy_reg = mii_phy_read(phy_addr, 1);
    if (!(phy_reg & 0x0004)) printf("Link is down\n");
    if (!(phy_reg & 0x0020)) printf("Auto-negotiation not complete\n");
    if (phy_reg & 0x0002) printf("Jabber condition detected\n");
    if (phy_reg & 0x0010) printf("Remote fault condition detected \n");

    if (known) {
	phy_reg = mii_phy_read(phy_addr, 25);
	if (phy_reg & 0x0004) {
	    phy_reg = mii_phy_read(phy_addr, 28);
	    printf("Phy operating at %d MBit/s in %s-duplex mode\n",
		phy_reg & 0x0002 ? 100 : 10,
		phy_reg & 0x0001 ? "full" : "half");
	}
	else
	    printf("bad link!!\n");
/*
left  off:no-link, green:100MBit, yellow:10MBit, blink:activity
right yellow:half-duplex, green:full-duplex
*/
//	mii_phy_write(phy_addr, 20, 0x0D52);
    }
}

unsigned short
mii_phy_read(unsigned short adr, unsigned short reg)
{
    int i;
    unsigned short tmp, val = 0;
	iopin_t phy_mdc;
	iopin_t phy_mdio;

	phy_mdc.port = 2; // Port C
	phy_mdc.pin = 19; // Port C
	phy_mdio.port = 2; // Port C
	phy_mdio.pin = 18; // Port C

    tmp = 0x6002 | (adr << 7) | (reg << 2);
	iopin_set_out(&phy_mdio); iopin_set_high(&phy_mdc); iopin_set_high(&phy_mdio);
    for (i = 0; i < 32; i++) {
	iopin_set_low(&phy_mdc); iopin_set_high(&phy_mdc);
    }
    for (i = 0; i < 16; i++) {
	iopin_set_low(&phy_mdc);
	if (tmp & 0x8000) iopin_set_high(&phy_mdio); // regs->bcsr8 |= MII_MDIO;
	else iopin_set_low(&phy_mdio); // regs->bcsr8 &= ~MII_MDIO;
	iopin_set_high(&phy_mdc);
	tmp <<= 1;
    }
	iopin_set_in(&phy_mdio);
    for (i = 0; i < 16; i++) {
	val <<= 1;
	iopin_set_high(&phy_mdc);
	if (iopin_is_high(&phy_mdio)) val |= 1;
	iopin_set_low(&phy_mdc);
    }
    return val;
}

void
mii_phy_write(unsigned short adr, unsigned short reg, unsigned short val)
{
    int i;
    unsigned short tmp;
	iopin_t phy_mdc;
	iopin_t phy_mdio;

	phy_mdc.port = 2; // Port C
	phy_mdc.pin = 19; // Port C
	phy_mdio.port = 2; // Port C
	phy_mdio.pin = 18; // Port C

    tmp = 0x5002 | (adr << 7) | (reg << 2);
	iopin_set_out(&phy_mdio); iopin_set_high(&phy_mdc); iopin_set_high(&phy_mdio);
    for (i = 0; i < 32; i++) {
	iopin_set_low(&phy_mdc); iopin_set_high(&phy_mdc);
    }
    for (i = 0; i < 16; i++) {
	iopin_set_low(&phy_mdc);
	if (tmp & 0x8000) iopin_set_high(&phy_mdio); // regs->bcsr8 |= MII_MDIO;
	else iopin_set_low(&phy_mdio); // regs->bcsr8 &= ~MII_MDIO;
	iopin_set_high(&phy_mdc);
	tmp <<= 1;
    }
    for (i = 0; i < 16; i++) {
	iopin_set_low(&phy_mdc);
	if (val & 0x8000) iopin_set_high(&phy_mdio); // regs->bcsr8 |= MII_MDIO;
	else iopin_set_low(&phy_mdio); // regs->bcsr8 &= ~MII_MDIO;
	iopin_set_high(&phy_mdc);
	val <<= 1;
    }
}

#endif

#else
void reset_phy (void) {};
void mii_discover_phy(int phy_addr) {};
#endif
